/* ---------------------------------------------------------------- */
/* Copyright (c) Atelier de BioInformatique			    */
/* @file: angle_io.h						    */
/* @desc: sequence IO in angle as fasta format / include file	    */
/*								    */
/* @history:							    */
/* @+	    <jompo> : Jul 02 : first version			    */
/* ---------------------------------------------------------------- */

#ifndef _H_angle_io

#define _H_angle_io

#ifndef _H_DTypes
#include "DTypes.h"		/* machine dependant types	*/
#endif

/* ==================================================== */
/* Constantes						*/
/* ==================================================== */

#define ANGLE_NAMLEN  256  	/* max length of seq. name	 */
#define ANGLE_COMLEN  512 	/* max length of seq. comment	 */


/* ==================================================== */
/* Macros standards					*/
/* ==================================================== */

#ifndef NEW
#define NEW(typ)		(typ*)malloc(sizeof(typ)) 
#define NEWN(typ, dim) 		(typ*)malloc((unsigned)(dim) * sizeof(typ))
#define REALLOC(typ, ptr, dim)	(typ*)realloc((void *) (ptr), (unsigned long)(dim) * sizeof(typ))
#define FREE(ptr)		free(ptr)
#endif

/* ==================================================== */
/* Structures de donnees				*/
/* ==================================================== */

typedef struct {			/* -- Sequence ---------------- */
	int	ok;			/* error flag			*/
	long 	length,			/* longueur			*/
		offset,			/* offset			*/
		bufsize;		/* size of current seq buffer	*/
	char    name[ANGLE_NAMLEN],	/* nom 				*/
		comment[ANGLE_COMLEN];	/* commentaire			*/
	INDX	*seq;			/* sequence			*/
} AngleSequence, *AngleSequencePtr;

/* ==================================================== */
/*  Prototypes (generated by mkproto)			*/
/* ==================================================== */

					/* libangle.c 	*/

char  		 *GetAngleName 	    ( char *buffer );
char 		 *GetAngleComment   ( char *buffer );

AngleSequencePtr NewAngleSequence   ( void );
AngleSequencePtr FreeAngleSequence  ( AngleSequencePtr seq );

void		CleanAngleSequence(AngleSequencePtr seq, INDX angmax);
SBool		 ReadAngleSequence  ( FILE *streamin, AngleSequencePtr seq , INDX angmax);
SBool		 GetAngleSequence   ( FILE *streamin, AngleSequencePtr seq , INDX angmax);
void 		 WriteAngleSequence ( FILE *streamou, AngleSequencePtr seq , INDX angmax);

void		 RewindAngleDB	    ( FILE *streamin );

#endif
