/*----------------------------------------------------------------------*/
/* ang2xyz_drive.c : driver pour ang2xyz				*/
/* purpose: genere un fichier pdb a partir d'un fichier d'angles	*/
/* de coordonnees internes alpha-tau-distance				*/
/*----------------------------------------------------------------------*/

/*----------------------------------------------------------------------*/
/*           INCLUDES     						*/
/*----------------------------------------------------------------------*/

#include "stdio.h"
#include "stdlib.h"
#include "string.h"
#include "math.h"
/* #if (defined  Darwin) || (defined  Linux) || (defined  IRIX) */
#include <unistd.h>             /* for getopt   */
/* #else */  /* No getopt.h file, lost ?*/
/* #define NO_GETOPT */
/* #include <getopt.h>              if getopt non on the machine */ 
/* #endif */

#include "libangles2xyz.h"

/*----------------------------------------------------------------------*/
/*           MACRO-DEFINES						*/
/*----------------------------------------------------------------------*/

#define GOOD		0
#define BAD		1
#define PHIPSI_FLAG	0
#define ALPHATAU_FLAG	1
#define ALPHA_FLAG	2
#define MAX_RES		2000  /*change : avant 1000*/
#ifndef NULL
#define NULL		0L
#endif

#define DEG2RAD		0.01745329252

/*----------------------------------------------------------------------*/
/*           TYPES  							*/
/*----------------------------------------------------------------------*/

int main(int argc, char **argv)
{
   char		line[256], inser, resname[MAX_RES][4];
   float	*(ang[3]);
   COORDS	cout[MAX_RES];
   int		nres[MAX_RES];
   int		i = 0, restot, carg, errflag = 0, angle_flag = ALPHATAU_FLAG;
#ifndef Darwin
   extern       char *optarg;
   extern       int optind, opterr;
#endif

#ifdef NO_GETOPT
   char         *ptr;
#endif
#ifndef Darwin
   extern       char *optarg;
   extern       int optind, opterr;
#endif
                                                                                                             
 
#ifdef NO_GETOPT
   for (i = 1; i < argc; i++) {         /* arguments                    */
      if (*argv[i] != '-') {
        if (i == argc-1) {
           if ((fil = fopen(argv[i], "r")) == NULL) /* error exit with 1        */
               myerror(strcat(strcpy(str, "cannot open file"), argv[i]), 1);
        }
        else
           myerror(strcat(strcpy(str,argv[i])," is not a valable argument"),5);
      }
      else {                            /* it is an argument            */
         ptr = argv[i]+1;
         while(*ptr) {
#else
   while ((carg = getopt(argc, argv, "paAh")) != -1) {
       switch (carg) {
#endif

                case 'p':               /* phipsi option        */
                    angle_flag = PHIPSI_FLAG;
                    break;

                case 'a':               /* alpha-tau option     */
                    angle_flag = ALPHATAU_FLAG;
                    break;

                case 'A':               /* alpha-tau option     */
                    angle_flag = ALPHA_FLAG;
                    break;

                case 'h':               /* help                 */
                case '?':               /* bad option           */
		    errflag++;
                    break;
#ifdef NO_GETOPT
           }
         }
      }
   }
#else  /* GETOPT */
        }

     }
#endif

     if (errflag || optind == 1) {
                fprintf(stderr, "%s takes an input angle file and output a xyz file\n",argv[0]);
                fprintf(stderr, " Usage: %s -[pah] < angle_file \n", argv[0]);
                fprintf(stderr, "     or %s -[pah] -A angle_file\n", argv[0]);
		fprintf(stderr, "          -p: phi-psi angle file\n");
		fprintf(stderr, "          -a: alpha-tau angle file\n");
		fprintf(stderr, "          -A: alpha angle file\n");
		fprintf(stderr, "          -h: this help :-)\n");
                exit(1);
  }

   ang[0] = (float *)malloc(MAX_RES*sizeof(*(ang[0])));
   ang[1] = (float *)malloc(MAX_RES*sizeof(*(ang[1])));
   ang[2] = (float *)malloc(MAX_RES*sizeof(*(ang[2])));

   if (ang[0] == NULL ||  ang[1] == NULL ||  ang[2] == NULL) {
      fprintf(stderr,"not enough memory, exiting...\n");
      exit(1);
   }
  
		/* les fichiers d'angles alpha sont ainsi:	*/
		/* NOMRES NRES ALPHA TAU DIST			*/
		/* ALPHA: angle entre ca-1,ca,ca+1,ca+2		*/
		/* TAU  : angle entre ca-1,ca,ca+1		*/
		/* DIST : distance entre ca,ca+1		*/
		/*						*/
		/* les fichiers d'angles phi-psi sont ainsi:	*/
		/* NOMRES NRES PHI PSI OMEGA			*/
		/* PHI  : angle entre C-1,N,CA,C		*/
		/* PSI  : angle entre N,CA,C,N+1		*/
		/* OMEGA: angle entre CA,C,N+1,CA+1		*/

   while (fgets(line, 255, stdin) != NULL) {
	 if (! strncmp("HEADER",line, 6) || ! strncmp("PATH",line, 4) || ! strncmp("NBRES",line, 5))
	   continue;
         if (sscanf(line,"%4s %d%c %f %f %f", (char *)&(resname[i]),
	 		&(nres[i]), &inser,
			&(ang[0][i]), &(ang[1][i]),
			&(ang[2][i])) != 6) {
	   fprintf(stderr,"erreur ligne %d <%s>;\nexiting...\n",i, line);
	   exit(1);
         }
         i++;
         if (i >= MAX_RES) {
	    fprintf(stderr,"too much residues, not enough memory, exiting...");
         }
   }

   restot = i;

   i = assign_coords(cout, ang, angle_flag, restot);

   for (i=0; i < restot; i++) {
      printf("ATOM %6d CA   %-3s %5d    %8.3f%8.3f%8.3f  1.00  0.00\n",
	  i+1, resname[i], nres[i], cout[i].x, cout[i].y, cout[i].z);
   }

   return 0;
}

