/*----------------------------------------------------------------------*/
/* ang2xyz_drive.c : driver pour ang2xyz				*/
/* purpose: genere un fichier pdb a partir d'un fichier d'angles	*/
/* de coordonnees internes alpha-tau-distance				*/
/*----------------------------------------------------------------------*/

/*----------------------------------------------------------------------*/
/*           INCLUDES     						*/
/*----------------------------------------------------------------------*/

#include "stdio.h"
#include "stdlib.h"
#include "string.h"
#include "math.h"
/* #if (defined  Darwin) || (defined  Linux) || (defined  IRIX) */
#include <unistd.h>             /* for getopt   */
/* #else */  /* No getopt.h file, lost ?*/
/* #define NO_GETOPT */
/* #include <getopt.h>              if getopt non on the machine */ 
/* #endif */

#include "libangles2xyz.h"

/*----------------------------------------------------------------------*/
/*           MACRO-DEFINES						*/
/*----------------------------------------------------------------------*/

#define GOOD		0
#define BAD		1
#define PHIPSI_FLAG	0
#define ALPHATAU_FLAG	1
#define ALPHA_FLAG	2
#define MAX_RES		1000
#ifndef NULL
#define NULL		0L
#endif

#define DEG2RAD		0.01745329252

/*----------------------------------------------------------------------*/
/*           TYPES  							*/
/*----------------------------------------------------------------------*/

/*----------------------------------------------------------------------*/
/*	FUNCTION PROTOTYPES						*/
/*----------------------------------------------------------------------*/


static int get_coo_from_angle(COORDS c0, COORDS c1, COORDS c2,
		COORDS *c3, float alpha, float tau, float normw);
static double tau_from_alpha(double alpha);
static int fit_dist_CA(COORDS c0, COORDS c1, COORDS c2,
		COORDS *c3, float distCA);

/*--------------------------------------------------------------*/
/* libangles2xyz.c   purpose: return a XYZ structure from		*/
/*             internal coordinates (Phi-Psi, or Alpha-Tau)	*/
/*--------------------------------------------------------------*/


/*--------------------------------------------------------------*/
/* retourne un tableau de coordonnees calculees d'apres         */
/* le tableau des angles "mean_ang" (phi-psi-omega ou           */
/* alpha-tau-distEntreCA ou alpha-x-distEntreCA)                  */
/*--------------------------------------------------------------*/
int assign_coords(COORDS *cmean, float **mean_ang,
			short phi_or_alpha, short len)
{
   int		i;
   COORDS	c0, cooC1, cooC2, cooN1, cooN2;
   double	distCA_C, distC_N, distN_CA, distCA_CA,
		angN_CA_C, angCA_C_N, angC_N_CA, tau;

		/* mean_ang[0] = angle phi ou alpha		*/
		/* mean_ang[1] = angle psi ou tau		*/
		/* mean_ang[2] = angle omega ou distCA-CA	*/



   switch(phi_or_alpha) {

      case PHIPSI_FLAG:
			/* ce sont les distances canoniques	*/
	distCA_C = 1.53;
	distC_N  = 1.32;
	distN_CA = 1.46;
	distCA_CA = 3.80;
		/* ce sont les angles canoniques	*/
	angC_N_CA  = 121.5;
	angN_CA_C  = 111.0;
	angCA_C_N  = 117.0;
	cooC1.x    = -1.0;
	cooC1.y    =  0.0;
	cooC1.z    =  0.0;
	cooN1.x    = -1.0;
	cooN1.y    = -1.0;
	cooN1.z    =  0.0;
   	cmean[0].x =  0.0;
   	cmean[0].y =  0.0;
   	cmean[0].z =  0.0;
			/* il faut :				*/
			/* 1) calculer CA-C			*/
			/* 2) calculer C-N			*/
			/* 3) calculer N-CA			*/

	for (i = 1; i < len; i++) {

				/* C (phi) 	*/
	      get_coo_from_angle(cooC1, cooN1, cmean[i-1],
	      		&cooC2 , mean_ang[0][i-1],
			angN_CA_C, distCA_C);

				/* N (psi)	*/
	      get_coo_from_angle(cooN1, cmean[i-1], cooC2,
			&cooN2, mean_ang[1][i-1],
			angCA_C_N, distC_N);

				/* CA (ome)	*/
	      get_coo_from_angle(cmean[i-1], cooC2, cooN2,
			&cmean[i], mean_ang[2][i-1],
			angC_N_CA, distN_CA);

	      fit_dist_CA(cmean[i-1], cooC2, cooN2, &cmean[i], distCA_CA);

	      cooC1 = cooC2;
	      cooN1 = cooN2;

	}

        break;

      case ALPHATAU_FLAG:
      case ALPHA_FLAG:
			/* meme chose que pour ALPHATAU_FLAG	*/
			/* mais on met les angles tau a ~110deg	*/

			/* a un CA est attache :		*/
			/* alpha entre CA-1,CA,CA+1,CA+2	*/
			/* tau   entre CA-1,CA,CA+1		*/
			/* mean_dist entre CA et CA+1		*/

			/* donc il faut passer pour le nouveau	*/
			/* CA, les trois precedents CA, avec	*/
			/* le alpha de CA-2 et le tau de CA-1	*/
			/* et la distance de CA-1		*/

			/* note: les distances se trouvent dans	*/
			/* mean_ang[2][....]			*/

   	c0.x =  0.0;
   	c0.y = -1.0;
   	c0.z =  0.0;
   	cmean[0].x = 0.0;
   	cmean[0].y = 0.0;
   	cmean[0].z = 0.0;
   	cmean[1].x = mean_ang[2][0];
   	cmean[1].y = 0.0;
   	cmean[1].z = 0.0;

	if (phi_or_alpha == ALPHATAU_FLAG) {

	   get_coo_from_angle(c0, cmean[0], cmean[1], &(cmean[2]),
		mean_ang[0][1], mean_ang[1][1], mean_ang[2][1]);

	   for (i = 3; i < len; i++)
	      get_coo_from_angle(cmean[i-3], cmean[i-2], cmean[i-1],
	      		&(cmean[i]),mean_ang[0][i-2], mean_ang[1][i-1],
			mean_ang[2][i-1]);

	}
	else {				 /* ALPHA only	*/

			/* tau moyen  = 106.5;		*/
			/* sigma=17.1873		*/
			/* ici, tau est calcule par rapport	*/
			/* a alpha, relation sinusoidale	*/

			/* de plus, il faut passer pour le nouveau	*/
			/* CA, les trois precedents CA, avec	*/
			/* le alpha de CA-2 et le tau de CA-1	*/
			/* et la distance de CA-1		*/
			/* note: tau de CA-1 est calcule sur le	*/
			/* alpha de CA-1			*/

			/* le premier diedre ne sert a rien...	*/
	   tau = tau_from_alpha((double)(mean_ang[0][1]<0.0?mean_ang[0][1]+360.0:mean_ang[0][1]));
/* fprintf(stderr,"tau_from_alpha=%f\n",tau); */
	   get_coo_from_angle(c0, cmean[0], cmean[1], &(cmean[2]),
		mean_ang[0][1], tau, mean_ang[2][1]);

	   for (i = 3; i < len; i++) {
	      tau = tau_from_alpha((double)(mean_ang[0][i-1]<0.0?mean_ang[0][i-1]+360.0:mean_ang[0][i-1]));
/* fprintf(stderr,"tau_from_alpha=%f\n",tau); */
	      get_coo_from_angle(cmean[i-3], cmean[i-2], cmean[i-1],
	      		&(cmean[i]), mean_ang[0][i-2], tau,
			mean_ang[2][i-1]);
	   }

	}

        break;

   }

   return GOOD;

}

/* ------------------------------------------------------------	*/
/* function tau of alpha					*/
/*  (note: alpha is in degree between [0,360]			*/
/* ------------------------------------------------------------	*/
static double tau_from_alpha(double alpha)
{
	/* sinusoidal approximation	*/
   /* return 108.86L-15.491L*sin(DEG2RAD*(0.98811L*alpha-335.26L)); */

	if (alpha >= 150) 
		return 52.7L+0.55044L*alpha-0.0010838*alpha*alpha;

	else	/* alpha < 150)	*/
		return 116.48L-0.91215L*alpha+0.010465L*alpha*alpha -
			0.000033355L*alpha*alpha*alpha;

}
/*--------------------------------------------------------------*/
/* calcule coords c3 a partir de c0,c1,c2 et des angles         */
/* alpha et tau et de la distance entre c2-c3                   */
/*--------------------------------------------------------------*/
static int get_coo_from_angle(COORDS c0, COORDS c1, COORDS c2,
		COORDS *c3, float alpha, float tau, float normw)
{
		/* c0, c1, c2 sont les coordonnees des trois	*/
		/* precedents CA, c3 sont les nouvelles coor-	*/
		/* donnees a calculer				*/
		/* normw est la norme du vecteur c2-c3 (dis-	*/
		/* tance entre les CA)				*/
		/* alpha et tau sont les angles de coordonnees	*/
		/* internes, et sont donc:			*/
		/* alpha est le diedre entre c0-c1-c2-c3	*/
		/* tau est l'angle entre c1-c2-c3		*/

   double	u1, u2, u3, v1, v2, v3,
		norm, pvuv1, pvuv2, pvuv3,
		pvvuv1, pvvuv2, pvvuv3,
		nsa, nca, nct;

   u1 = (double)(c1.x - c0.x);
   u2 = (double)(c1.y - c0.y);
   u3 = (double)(c1.z - c0.z);

   v1 = (double)(c2.x - c1.x);
   v2 = (double)(c2.y - c1.y);
   v3 = (double)(c2.z - c1.z);

			/* normalisation de v		*/

   norm = sqrt(v1*v1+v2*v2+v3*v3);
   v1 /=  norm;
   v2 /=  norm;
   v3 /=  norm;

			/* pvuv: prod vect (u*v)	*/

   pvuv1 = u2*v3 - u3*v2;
   pvuv2 = u3*v1 - u1*v3;
   pvuv3 = u1*v2 - u2*v1;

			/* normalisation de pvuv	*/

   norm = sqrt(pvuv1*pvuv1+pvuv2*pvuv2+pvuv3*pvuv3);
   pvuv1 /= norm;
   pvuv2 /= norm;
   pvuv3 /= norm;

			/* pvvuv: prod vect u*(u*v)	*/

   pvvuv1 = v2*pvuv3 - v3*pvuv2;
   pvvuv2 = v3*pvuv1 - v1*pvuv3;
   pvvuv3 = v1*pvuv2 - v2*pvuv1;
   			
			/* normalisation de pvvuv	*/

   norm = sqrt(pvvuv1*pvvuv1+pvvuv2*pvvuv2+pvvuv3*pvvuv3);
   pvvuv1 /= norm;
   pvvuv2 /= norm;
   pvvuv3 /= norm;

   nca = cos(DEG2RAD * alpha);
   nsa = sin(DEG2RAD * alpha);
   nct = tan(DEG2RAD * (tau-90));

   u1 = nca*(-pvvuv1) + nsa*pvuv1 + v1*nct;
   u2 = nca*(-pvvuv2) + nsa*pvuv2 + v2*nct;
   u3 = nca*(-pvvuv3) + nsa*pvuv3 + v3*nct;

			/* renormalise le vecteur	*/
			/* pour avoir normw		*/

   norm = sqrt(u1*u1+u2*u2+u3*u3);
   u1 = u1*normw/norm;
   u2 = u2*normw/norm;
   u3 = u3*normw/norm;
			/* calcul du 4eme point		*/
   c3->x = u1 + c2.x;
   c3->y = u2 + c2.y;
   c3->z = u3 + c2.z;

   return GOOD;
}
/*--------------------------------------------------------------*/
/* reajuste la position du deuxieme carbone alpha		*/
/* en gardant les angles					*/
/*--------------------------------------------------------------*/
static int fit_dist_CA(COORDS c0, COORDS c1, COORDS c2,
		COORDS *c3, float distCA)
{
   double	norm,diff,u1,u2,u3,v1,v2,v3,w1,w2,w3,t1,t2,t3;

   u1 = c1.x - c0.x; u2 = c1.y - c0.y; u3 = c1.z - c0.z;
   v1 = c2.x - c1.x; v2 = c2.y - c1.y; v3 = c2.z - c1.z;
   w1 = c3->x - c2.x; w2 = c3->y - c2.y; w3 = c3->z - c2.z;

   t1 = u1+v1+w1;
   t2 = u2+v2+w2;
   t3 = u3+v3+w3;

   norm = sqrt(t1*t1+t2*t2+t3*t3);

   diff = distCA - norm;

/*
	marche bien, simplification au dessous
   norm = diff/(sqrt(u1*u1+u2*u2+u3*u3+2*u1*v1+v1*v1+2*u2*v2+v2*v2+2*u3*v3+v3*v3
	+2*u1*w1+2*v1*w1+w1*w1+2*u2*w2+2*v2*w2+w2*w2+2*u3*w3+2*v3*w3+w3*w3));
 */
   norm = diff/(sqrt(u1*(u1+2*v1+2*w1)+u2*(u2+2*v2+2*w2)+u3*(u3+2*w3+2*v3)
	+v1*(v1+2*w1)+v2*(v2+2*w2)+v3*(v3+2*w3)
	+w1*w1+w2*w2+w3*w3));

   t1 += norm*(u1+v1+w1);
   t2 += norm*(u2+v2+w2);
   t3 += norm*(u3+v3+w3);

   c3->x = t1 + c0.x;
   c3->y = t2 + c0.y;
   c3->z = t3 + c0.z;

   return GOOD;

}
