/* -------------------------------------------------------------------- */
/* constants								*/
/* -------------------------------------------------------------------- */

#define NO_ANGLE	500	/* invalid angle		*/
#define NO_COORDINATE -16777216.0     /* 2^24 invalid coordinate */
#define NO_CHAIN	' '	/* ' ' == pas de chaine		*/
#define NO_INSER	' '	/* ' ' == pas d'insertion	*/

#define ATOMNAME	6	/* length + 1 of atom name	*/
#define RESNAME		4	/* length + 1 of residue name	*/

#define HEADERSIZE	46	/* total size of header in 	*/
				/* binary format		*/

				/* maximum distance for two	*/
				/* residues to be neighbours 	*/
#define DIST_MAX_BETWEEN_NEIGHBOUR_RESIDUE  5
#define MAX_NEIGHBOURS	50	/* maximum neighbour for a residue	*/

/*
#define MINPDBNUM	-2147483648
#define MAXPDBNUM	2147483647
*/
#define MINPDBNUM	-32768
#define MAXPDBNUM	32767


#define NOCHAIN_CHAR	'@'	/* not a possible chain character	*/

#ifdef IRIX
#include <limits.h>             /* for MAX_PATH  */
#endif /* IRIX */
#ifdef SunOS
#include <limits.h>             /* for MAX_PATH  */
#endif /* SunOS */
#ifdef Darwin
#include <sys/syslimits.h> /* for MAX_PATH   */
#endif
#ifdef Linux
#include <linux/limits.h>       /* for MAX_PATH */
#endif /* Linux */


typedef union {
   short num[2];      /* num[1] is the PDB number             */
   char  inser[4];    /* inser[0] is the inser character      */
} PdbNum;

/* -------------------------------------------------------------------- */
/* structures								*/
/* -------------------------------------------------------------------- */

typedef struct {
  char resName[4];
  PdbNum numPDB;
  float alpha, tau, dist;
  float  x, y, z;
  char chain;
} BinaryRes;


typedef struct {			/* -- atome (long desc)		*/
	float		x, y, z;	/* cartesian coordinates	*/
	float		localize;	/* occupancy			*/
	float		tfactor;	/* temperature factor		*/
	short		num;		/* atom number in pdb file	*/
	short		ires;		/* numero du residu		*/
	char		name[ATOMNAME];	/* atom name			*/
	char		resn[RESNAME];	/* atom residue name		*/
	char		chain;		/* chaine de la proteine	*/
	char		inser;		/* residu insere		*/
} Atom;

typedef struct _ratom {			/* -- atom (short desc)		*/
	float		x, y, z;	/* cartesian coordinates	*/
	struct _ratom	*next;		/* next in list			*/
	float		tfactor;	/* temperature factor		*/
	char		name[ATOMNAME];	/* atom name			*/
} ResAtom;

typedef struct _residu {		/* -- residue 			*/
	ResAtom		*first_atom;	/* first atom in residue	*/
	ResAtom		*last_atom;	/* last atom in residue		*/
	float		phi, psi, ome;	/* phi, psi & omega dihed.	*/
	float		alpha, tau;	/* alpha, tau (Levitt 1976 )	*/
	float		x1, x2, x3;	/* chi1, ch2 & chi3 dihed.	*/
	float		x, y, z;	/* geometric center		*/
        float           dis13;          /* distance between CA-1 & CA+1	*/
        float           dis12;          /* distance between CA & CA+1	*/
					/* array of neighbours		*/
	struct _residu	*neighbour[MAX_NEIGHBOURS];
	struct _residu	*prev;		/* previous in list		*/
	struct _residu	*next;		/* next in list			*/

	short		nb_neighbours;	/* number of neighbours		*/
	short		num;		/* number in the whole molecule	*/
	short		pdbnum;		/* number in the pdb file	*/
	short		dummy;		/* for structure alignment	*/

	char		name[RESNAME];	/* residue name	(3 letters)	*/
	char		chain;		/* chaine de la proteine	*/
	char		inser;		/* residu insere		*/
} Residu, *PtrResidu;

/* -------------------- */
/* local structure	*/
/* -------------------- */
typedef struct tyChain{
  Residu *pChain;
  int nbRes;
}tyChain;


/* -------------------------------------------------------------------- */
/* prototypes           						*/
/* -------------------------------------------------------------------- */

int	set_residue_geocenter(Residu *r);
int	set_residue_neighbours(Residu *res, Residu *first);
float	get_dist_max_from_geocenter(Residu *r);
Residu	*readPdbResidues(FILE *f, char *header, int *nbres);
Residu	*readPdbResiduesChain(FILE *f, tyChain **tChain, char *name, int *nbChain);
/* ---------------------------------------------------- */
/* ReadPhipsiLine Lit une ligne au format de sortie de	*/
/* phipsi -a [-c] 					*/
/* retourne -1 si probleme, 0 sinon                     */
/* ---------------------------------------------------- */
void MyError(FILE *stream, int stat, char *fmt, ...);
int	ReadPhipsiLine(FILE *pFi, BinaryRes *pRes, int verbose);
int	writeBinaryFile(FILE *fout, char *header, char *path, int nb_res_tot,
		Residu *first_res, int verbose, char chain, 
		int minpdbnum, int maxpdbnum);
BinaryRes *readBinary(FILE *f, char *header, char *path, int *nbres, int verbose);
void	free_list_residus(Residu *first_res);
int	get_phi_psi(Residu *r, int verbose);
int	get_chi(Residu *r, int verbose);
int     get_distca_1_3( Residu *r, int verbose);
int     get_distca_1_2( Residu *r, int verbose);
int	get_alpha_tau(Residu *r, int verbose);
int	get_CA_xyz(Residu *r, float *x, float *y, float *z, int verbose);
int	get_atom(char *line, Atom *atom);
void	pad_with_blank(char *h, int totsize);
float	get_CA_TFactor(Residu *r,  int verbose);

