/* -------------------------------------------------------------------- */
/* Yacool / Gloup / Jul 90						*/
/* Yacool Extensions / To be added into Yacool V1.0 /			*/
/* @File Nmatrix.c							*/
/* New matrix Library							*/
/* ------------------------------------------------------------------------ */
/* -------------------------------------------- */
/* librairie acces aux matrices					*/
/* -------------------------------------------- */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <math.h>
#include <limits.h>
#include <stdarg.h>
#include "Nmatrix.h"
#include "macros.h"
#include "util.h"


		/* proba of amino acids in SwissProt	*/

static double sProbaSwissProt[26] = {0.0773/*A*/,0.0/*B*/,0.0153/*C*/,0.0508/*D*/,0.0605/*E*/,0.0413/*F*/,0.0691/*G*/,0.0227/*H*/,0.0599/*I*/,0.0/*J*/,0.0555/*K*/,0.0972/*L*/,0.0239/*M*/,0.0452/*N*/,0.0/*O*/,0.0494/*P*/,0.0387/*Q*/,0.0533/*R*/,0.0707/*S*/,0.0573/*T*/,0.0/*U*/,0.0647/*V*/,0.0138/*W*/,0.0/*X*/,0.0316/*Y*/,0.0/*Z*/};

typedef struct {
	int		from, to;
	Boolean modify_diags;
}mat_adj_info;

#if 0
static void ask_adj_info(pam_matrix *matrix, mat_adj_info *info);
static char *mat_type_string(int type);
#endif

/* -------------------------------------------- */
/* variables statiques							*/
/* -------------------------------------------- */

static float sIndelOpeRatio = GAP_OPEN_RATIO;
static float sIndelExtRatio = GAP_EXT_RATIO;
static float sIndelGapGapRatio = GAP_GAP_RATIO;

static int sMatrixMaxVal = MAT_MAXVAL;
static int sMatrixMinVal = MAT_MINVAL;

/* -------------------------------------------- */
/* fonctions internes							*/
/* -------------------------------------------- */

#if 0
/* -------------------------------------------- */
/* @ifunction :	mat_type_string					*/
/* convert type -> string						*/
/* -------------------------------------------- */

static char *mat_type_string(int type)
{
	static char	buffer[32];

	switch (type) {
		case MAT_PAM_TYPE :
			strcpy(buffer, "Pam");
			break;
		case MAT_PROX_TYPE :
			strcpy(buffer, "Proximity");
			break;
		case MAT_DIST_TYPE :
			strcpy(buffer, "Distance");
			break;
		default :
			strcpy(buffer, "Unknown");
			break;
	}
	
	/* <jompo> added return value */
	return buffer;
}
#endif

#if 0
/* -------------------------------------------- */
/* @ifunction :	ask_adj_info					*/
/* dialog for adjustment parameters				*/
/* -------------------------------------------- */

static void ask_adj_info(pam_matrix *matrix, mat_adj_info *info)
{

						/* default values		*/
	info->modify_diags = vrai;

	return;
}
#endif
	

/* -------------------------------------------- */
/* @Function void unitary_matrix				*/
/* built unitary matrix 		 				*/
/* -------------------------------------------- */
void unitary_matrix(pam_matrix *matrix, int type, char *vocab)
{
	int 	i, j, size, diag, out;


	strcpy(matrix->name, "Unitary");
	strcpy(matrix->vocab, vocab);

	size = strlen(vocab);
	
	matrix->pathname[0] = '\000';
	
	matrix->type      = type;
	matrix->size 	  = size;

	switch (type) {
	
		case MAT_DIST_TYPE :
				diag = MAT_MINVAL;
				out  = MAT_MAXVAL;
				matrix->threshold = MAT_MAXVAL;
				matrix->min_val = MAT_MINVAL;
				matrix->max_val = MAT_MAXVAL;
				break;	

		case MAT_PAM_TYPE  :
				matrix->threshold = MAT_PAM_MAXVAL;
				matrix->min_val = MAT_PAM_MINVAL;
				matrix->max_val = MAT_PAM_MAXVAL;
				diag = MAT_PAM_MAXVAL;
				out  = MAT_PAM_MINVAL;
				break;	

		case MAT_PROX_TYPE :
		default :
				matrix->threshold = MAT_MAXVAL;
				matrix->min_val = MAT_MINVAL;
				matrix->max_val = MAT_MAXVAL;
				diag = MAT_MAXVAL;
				out  = MAT_MINVAL;
				break;	

	}

	for (i = 0 ; i < size ; i++) {
	   for (j = i+1 ; j < size ; j++) {
			matrix->val[j][i]    = matrix->val[i][j]    = out;
			matrix->classe[j][i] = matrix->classe[i][j] = 0;
			matrix->freq[j][i] = matrix->freq[i][j] = 0.0L;
		}
		matrix->val[i][i] = diag;
		matrix->classe[i][i] = 1;
		matrix->freq[i][i] = 1.0L;
		matrix->weight[i] = MAT_MAXVAL;
	}
}

/* -------------------------------------------- */
/* @Function Boolean my_alert()		*/
/* -------------------------------------------- */
void my_alert(FILE *stream, char *fmt, ...)
{
   va_list args;

                                /* give a message on stream "stream"    */
   va_start(args,fmt);
   vfprintf(stream, fmt, args);
   va_end(args);
   fflush(stream);

                                /* jompo A FINIR: a voir si on veut mettre sur
                                   un socket
                                 */

   return;
}

/* -------------------------------------------- */
/* @Function Boolean load_matrix()		*/
/* loading matrix file		 		*/
/* -------------------------------------------- */
Boolean load_matrix(pam_matrix *matrix, char *fname)
 {
	int		i, j, lvocab = 0;
	Boolean		read_ok;
	char		c[2], line[256];
	FILE		*filin;
	
						
	read_ok = faux;				/* assume error		*/


	strcpy(matrix->pathname, fname);
	strcpy(matrix->name, fname);
	
	
	filin = fopen(matrix->pathname, "r");	/* open file		*/
	
	if (! filin) {				/* missed !		*/
		return faux;
	}		
	
 

							/* lecture matrice			*/	
							/* 1er char = '/'			*/
	*c = fgetc(filin);
	read_ok = (*c == '/');				/* Smarties like PAM matrix	*/

	if (!read_ok) {
							/* Blosum like matrix ?	*/

	   rewind(filin);
	   read_ok = read_blosum_matrix(filin, matrix);
	   fclose(filin);
	   return read_ok;
	}

	if (read_ok) {
						/* raz matrices				*/
		for (i = 0 ; i < MAT_SIZE_MAX ; i++) {
	
			for (j = i ; j < MAT_SIZE_MAX ; j++) {
				matrix->val[i][j] = matrix->val[j][i] = 0;
				matrix->classe[i][j] = matrix->classe[j][i] = 0;
				matrix->freq[i][j] = matrix->freq[j][i] = 0.0L;
			}
	
			matrix->weight[i] = 0;
		} 
	}

							/* skip info				*/
	if (read_ok) {
		do {		
		   *c = fgetc(filin);
		} while ((*c != '/') && (*((int *)c) != EOF));
		 
		read_ok = (*c == '/');
	}
		

							/* key(int), threshold(int)	*/
	if (read_ok)
		read_ok = fscanf(filin, "%d%d", &(matrix->type), &(matrix->threshold)) == 2; 
	
	
							/* vocabulary				*/
							
	if (read_ok) {
		read_ok = fscanf(filin, "%s", matrix->vocab) != EOF;
		matrix->size = lvocab = strlen(matrix->vocab);
	}
							/* matrix					*/
	
	if (read_ok) {
		for (i = 0 ; (i < lvocab) && read_ok ; i++) {
		  for (j = 0 ; (j < lvocab) && read_ok ; j++)
		     read_ok = fscanf(filin, "%d", &(matrix->val[i][j])) == 1;
	          read_ok = (fgets(line, 256, filin) != NULL);	/* read end of line	*/
		}
	}

							/* store diag. elements		*/
	if (read_ok)
		for (i = 0 ; i < lvocab ; i++)
			switch (matrix->type) {
				case MAT_DIST_TYPE :
						matrix->weight[i] = MAT_MAXVAL - matrix->val[i][i];
						break;	
		
				case MAT_PAM_TYPE  :
				case MAT_PROX_TYPE :
				default :
						matrix->weight[i] = matrix->val[i][i];
						break;	
			}

							/* classes					*/				
	if (read_ok) {
		for (i = 0 ; (i < lvocab) && read_ok ; i++)
		  for (j = 0 ; (j < lvocab) && read_ok ; j++)
		     read_ok = fscanf(filin, "%d", &(matrix->classe[i][j])) == 1;
	}

	fclose(filin);

 	return read_ok;
 }



/* -------------------------------------------- */
/* read a matrix in filin FILE			*/
/* PAM/BLOSUM like matrix				*/
/* -------------------------------------------- */
Boolean read_blosum_matrix(FILE *filin, pam_matrix* matrix)
{
char		line[256], *s, *ss;
Boolean		read_ok;
int		i, j, lvocab = 0;

						/* raz matrices				*/
   for (i = 0 ; i < MAT_SIZE_MAX ; i++) {
	
      for (j = i ; j < MAT_SIZE_MAX ; j++) {
         matrix->val[i][j] = matrix->val[j][i] = 0;
         matrix->classe[i][j] = matrix->classe[j][i] = 0;
	 matrix->freq[i][j] = matrix->freq[j][i] = 0.0L;
      }
	
      matrix->weight[i] = 0;
   } 
   

   do {
	read_ok = (fgets(line, 256, filin) != NULL);
   } while (read_ok && line[0] == '#');

#if 0	/* old */
 {
 float		wdel, wldel;
   if (read_ok) {
      if (sscanf(line, "%f %f", &wdel, &wldel) != 2) {	/* gap weight, gap lengthening weight */
         return faux;
      }
   }
 } 
#endif

   matrix->min_val = INT_MAX;		/* max int value	*/
   matrix->max_val = INT_MIN;		/* min int value	*/
   matrix->type = MAT_PAM_TYPE;
   matrix->threshold = 0.0;			/* arbitrairement a 0	*/

					/* if first line is vocabulary	*/
 
   if (read_ok) {
      if (line[strlen(line)-1] == '\n')
		line[strlen(line)-1] = '\0';

			/* trim space	*/
      s = ss = line;
      while(*s) {
	 if (*s == ' ')
	   s++;
	 else {
	   *ss=*s;
           ss++;s++;
	 }
      }
			/* end of string	*/
      *ss = *s;

      if (strlen(line) > 27) {
         read_ok = faux;
      }
      else {
	 strcpy(matrix->vocab, line);
         matrix->size = lvocab = strlen(matrix->vocab);
      }
   }


				/* matrix					*/
   if (read_ok) {
	for (i = 0 ; (i < lvocab) && read_ok ; i++) {
	  for (j = 0 ; (j < lvocab) && read_ok ; j++) {
	     read_ok = fscanf(filin, "%d", &(matrix->val[i][j])) == 1;
/*	     fprintf(stderr, "%d  ", matrix->val[i][j]);*/
	     if (matrix->val[i][j] < matrix->min_val)
		matrix->min_val = matrix->val[i][j];
	     if (matrix->val[i][j] > matrix->max_val)
		matrix->max_val = matrix->val[i][j];
	  }
	  read_ok = (fgets(line, 256, filin) != NULL);	/* read end of line	*/
	}

			/* rallonge la sauce pour les symboles autres	*/
      if (lvocab == 21) {		/* tous les aa + X	*/

         matrix->vocab[21] = 'C';
         matrix->vocab[22] = 'I';
         matrix->vocab[23] = 'O';
         matrix->vocab[24] = 'U';
         matrix->vocab[25] = 'Z';
         matrix->vocab[26] = '\0';
         matrix->size = lvocab = strlen(matrix->vocab);
  
         for (i = 21; i < 26; i++) {
	    for (j = 0; j < 26; j++) {
	       matrix->val[i][j] = matrix->val[j][i] = matrix->min_val;
	    }
         }
      }
      else if (lvocab == 24) {	/* typical BLOSUM 	*/
				/* alphabet:ARNDCQEGHILKMFPSTWYVBZX* */
         matrix->vocab[23] = 'J';	/* on remplace le "*"	*/
         matrix->vocab[24] = 'O';
         matrix->vocab[25] = 'U';
         matrix->vocab[26] = '\0';
         matrix->size = lvocab = strlen(matrix->vocab);
  
         for (i = 23; i < 26; i++) {
	    for (j = 0; j < 26; j++) {
	       matrix->val[i][j] = matrix->val[j][i] = matrix->min_val;
	    }
         }
      }
#if 0 /* debug */

printinfo_matrix(stderr, matrix, "JOMPO", vrai, vrai);

#endif
							/* store diag. elements		*/
	for (i = 0 ; i < lvocab ; i++)
		switch (matrix->type) {
		   case MAT_DIST_TYPE :
			matrix->weight[i] = MAT_MAXVAL - matrix->val[i][i];
			break;	
		
		   case MAT_PAM_TYPE  :
		   case MAT_PROX_TYPE :
		   default :
			matrix->weight[i] = matrix->val[i][i];
			break;	
		}

   
      return vrai;
   }
 
   return faux;
}

/* -------------------------------------------- */
/* @Function Boolean reorder_matrix		*/
/* reorder matrix to a std vocab		*/
/* -------------------------------------------- */
Boolean reorder_matrix(matrix, std_vocab)
	pam_matrix  *matrix;
	char 		*std_vocab;
{
	int 	i, j, cpos;
	int	val;
	Boolean	ok = faux;
	char	c, *cc;
	double	d;
	
	if (! std_vocab)
		return faux;
					/* -------------------- */
					/* reorder matrix	*/
					/* -------------------- */
							
	for (i = 0 , cc = std_vocab ; *cc ; i++, cc++) {
	
		cpos = strchr(matrix->vocab, *cc) - matrix->vocab;
		if (! (ok = (cpos >= 0)))
			break;

		if (cpos == i) 
			continue;
			
							/* swap vocabulary			 */

		c           		= matrix->vocab[i];
		matrix->vocab[i]    = matrix->vocab[cpos];
		matrix->vocab[cpos] = c;

							/* swap diag				 */
		val 			     = matrix->weight[i];
		matrix->weight[i]    = matrix->weight[cpos];
		matrix->weight[cpos] = val;

							/* swap rows				 */

		for (j = 0 ; j < matrix->size ; j++) {
			val 				    = matrix->val[i][j];
			matrix->val[i][j]	    = matrix->val[cpos][j];
			matrix->val[cpos][j]    = val;
			d 				    = matrix->val[i][j];
			matrix->freq[i][j]	    = matrix->freq[cpos][j];
			matrix->freq[cpos][j]    = d;
			val 				    = matrix->classe[i][j];
			matrix->classe[i][j]    = matrix->classe[cpos][j];
			matrix->classe[cpos][j] = val;			
		}
							/* swap columns			 */
			
		for (j = 0 ; j < matrix->size ; j++) {
			val 				    = matrix->val[j][i];
			matrix->val[j][i]	    = matrix->val[j][cpos];
			matrix->val[j][cpos]    = val;
			d 				    = matrix->freq[j][i];
			matrix->freq[j][i]	 = matrix->freq[j][cpos];
			matrix->freq[j][cpos]    = d;
			val 				    = matrix->classe[j][i];
			matrix->classe[j][i]    = matrix->classe[j][cpos];
			matrix->classe[j][cpos] = val;			
		}

	}

	return ok;
}

/* -------------------------------------------- */
/* @Function void printinfo_matrix		*/
/* output matrix to a device			*/
/* -------------------------------------------- */
void printinfo_matrix(FILE *dev, pam_matrix *matrix, char *head, 
			Boolean put_values, Boolean put_classes)
{
	int	i, j;
	char	*cc;
	
	switch (matrix->type) {
		case MAT_PAM_TYPE :
			my_alert(dev, "%s PAM ", head);
			break;
		case MAT_PROX_TYPE :
			my_alert(dev, "%s Proximity ", head);
			break;
		case MAT_DIST_TYPE :
			my_alert(dev, "%s Distance ", head);
			break;
	}						 

	my_alert(dev, "Matrix used : %s\n", matrix->name);
	my_alert(dev, "%s Vocabulary : %s\n", head, matrix->vocab);
	my_alert(dev, "%s Threshold : %d\n", head, matrix->threshold);
	
	if (put_values) {

		my_alert(dev, "%s Values :\n%s    ", head, head);
		for (cc = matrix->vocab ; *cc ; cc++)
			my_alert(dev, "   %c", *cc);
		my_alert(dev, "\n");
		
		for (i = 0, cc = matrix->vocab ; *cc ; i++, cc++) {
			my_alert(dev, "%s %c   ", head, *cc);
			for (j = 0; j < matrix->size;  j++)
				my_alert(dev, "%3d ", matrix->val[i][j]);
			my_alert(dev, "\n");
		}
	}
	
	if (put_classes) {

		my_alert(dev, "%s Classes :\n%s   ", head, head);
		for (cc = matrix->vocab ; *cc ; cc++)
			my_alert(dev, "%c ", *cc);
		my_alert(dev, "\n");
		
		for (i = 0, cc = matrix->vocab ; *cc ; i++, cc++) {
			my_alert(dev, "%s %c ", head, *cc);
			for (j = 0; j < matrix->size;  j++)
				my_alert(dev, "%c ", (matrix->classe[i][j] ? 'o' : '.'));
			my_alert(dev, "\n");
		}
	}
	
	
}

/* -------------------------------------------- */
/* @Function void printinfo_pseudomatrix	*/
/* output pseudo_matrix to a device		*/
/* -------------------------------------------- */
void printinfo_pseudomatrix(FILE *dev, pseudo_matrix *pmatrix, char *head, 
			Boolean put_values)
{
	int	i, j;
	char	*cc;
	
	switch (pmatrix->type) {
		case MAT_PAM_TYPE :
			my_alert(dev, "%s PAM ", head);
			break;
		case MAT_PROX_TYPE :
			my_alert(dev, "%s Proximity ", head);
			break;
		case MAT_DIST_TYPE :
			my_alert(dev, "%s Distance ", head);
			break;
	}						 

	my_alert(dev, "%s Vocabulary : %s\n", head, pmatrix->vocab);
	
	if (put_values) {

		my_alert(dev, "%s Values :\n%s    ", head, head);
		for (cc = pmatrix->vocab ; *cc ; cc++)
			my_alert(dev, "   %c", *cc);
		my_alert(dev, "\n");
		
		for (i = 0, cc = pmatrix->vocab ; *cc ; i++, cc++) {
			my_alert(dev, "%s %c   ", head, *cc);
			for (j = 0; j < pmatrix->size;  j++)
				my_alert(dev, "%3d ", pmatrix->val[i][j]);
			my_alert(dev, "\n");
		}
	}

	return;	
}

/* -------------------------------------------- */
/* @Function Boolean write_matrix()				*/
/* write matrix to file		 					*/
/* -------------------------------------------- */
Boolean write_matrix(char *outname, pam_matrix *matrix)
 {
	int			i, j, lvocab;
	FILE		*streamou;
	char		path[256];
	


	strcpy(path, outname);
		
	streamou = fopen(path, "w");
	
	if (! streamou)
		return faux;

							/* writing begins here		*/
		
							/* key(int), threshold(int)	*/
	fprintf(streamou, "%d %d\n", matrix->type, matrix->threshold);
	
							/* vocabulary				*/
	fprintf(streamou, "%s\n", matrix->vocab);

							/* matrix					*/
	lvocab = matrix->size;
	for (i = 0 ; i < lvocab ; i++) {
	   for (j = 0 ; j < lvocab ; j++)
	       fprintf(streamou, "%3d", matrix->val[i][j]);
	   fprintf(streamou, "\n");
	}
							/* classes					*/				
	for (i = 0 ; i < lvocab ; i++) {
	    for (j = 0 ; j < lvocab ; j++)
	        fprintf(streamou, "%3d", matrix->classe[i][j]);
	   fprintf(streamou, "\n");
	}

	for (i = 0 ; i < lvocab ; i++) {
	    for (j = 0 ; j < lvocab ; j++)
	        fprintf(streamou, "%4.2f ", matrix->freq[i][j]);
	   fprintf(streamou, "\n");
	}

	fclose(streamou);


	return vrai;
}

/* -------------------------------------------- */
/* @Function Boolean diag_verify()				*/
/* verify diags of a matrix		 				*/
/* -------------------------------------------- */
Boolean diag_verify(matrix)
	pam_matrix	*matrix;
{
	int 	i;
	int	diagval;
	 		
	 		
	switch (matrix->type) {
	
		case MAT_PAM_TYPE :
			return vrai;	
			
		case MAT_PROX_TYPE :
			diagval = MAT_MAXVAL;
			break;	
			
		case MAT_DIST_TYPE :
			diagval = MAT_MINVAL;
			break;	
			
		default :
			return faux;
	}	

	for (i = 0 ; i < matrix->size ; i++)
		if (matrix->val[i][i] != diagval)
			return faux;

	return vrai;
}

/* -------------------------------------------- */
/* @Function Boolean sym_verify()				*/
/* verify symetry of a matrix		 			*/
/* -------------------------------------------- */
Boolean sym_verify(matrix)
	pam_matrix	*matrix;
{

	int 	i, j;

	for (i = 0 ; i < matrix->size ; i++)
	for (j = i+1 ; j < matrix->size ; j++)
		if (matrix->val[i][j] != matrix->val[j][i])
			return faux;
			
	return vrai;
}

/* -------------------------------------------- */
/* @Function Boolean val_verify()				*/
/* verify values of a matrix		 			*/
/* -------------------------------------------- */
Boolean val_verify(pam_matrix *matrix)
{

	int 	i, j;

	for (i = 0 ; i < matrix->size ; i++)
	for (j = 0 ; j < matrix->size ; j++)
		if ((matrix->val[i][j] > matrix->max_val) ||
			 (matrix->val[i][j] < matrix->min_val))
			return faux;
			
	return vrai;
}

/* -------------------------------------------- */
/* @Function Boolean simple_verify()			*/
/* simple verifications of a matrix		 		*/
/* -------------------------------------------- */
Boolean simple_verify(pam_matrix *matrix, Boolean diagtoo)
 {
	Boolean		ok,ok1,ok2;
	
	ok1 = sym_verify(matrix);
	if (! ok1) 
	   fprintf(stderr,"Matrix file is not symetric\n");

        ok2 = val_verify(matrix);
	if (! ok2) 
	   fprintf(stderr,"Matrix values < minval or > maxval\n");

	ok = ok1 && ok2 && ((! diagtoo) || diag_verify(matrix));
	
	return ok;
}

/* -------------------------------------------- */
/* @Function Boolean complain_pam_matrix()		*/
/* complains about PAM matrix					*/
/* -------------------------------------------- */
Boolean complain_pam_matrix(pam_matrix *matrix)
{
	if (matrix->type == MAT_PAM_TYPE)
		fprintf(stderr, "Bad matrix (similarity should be distance)\n");

	return vrai;
}

/* -------------------------------------------- */
/* @Function Boolean complain_fmt_matrix()		*/
/* complains about matrix format				*/
/* -------------------------------------------- */
Boolean complain_fmt_matrix(matrix)
	pam_matrix	*matrix;
{
	fprintf(stderr, "bad type matrix\n");
	return vrai;
}
	
/* -------------------------------------------- */
/* @Function Boolean simple_convert()		*/
/* simple conversion of a matrix 		*/
/* -------------------------------------------- */
Boolean simple_convert(pam_matrix *matrix, int totype, int verbose, int convert_method)
{
	int 	i, j, k, n;
	double	dist, ratio, m;
	float	range;


	if (matrix->type == totype)
		return vrai;


						/* go on with conversion		*/

	if ((matrix->type == MAT_PROX_TYPE || matrix->type == MAT_PAM_TYPE) && totype == MAT_DIST_TYPE) {
	   if (convert_method == CHI2_METHOD) {
	      chi2_convert(matrix);
	   }
	   else if (convert_method == SIMPLEDIST_METHOD) {
				/* calcul par la formule dist =		*/
				/* 1-(val-min)/(max-min)		*/

				/* i.e. distance(i,j) is 1-((prox(i,j)-min)/(max-min))	*/
				/* moyenne la diagonale pour eviter	*/
				/* que le haut score du trp/trp ne	*/
				/* fausse le tout			*/
		m = 0.L;
		for (n=0, i=0 ; i < matrix->size ; i++) {
		   if (matrix->val[i][i] > 0) {
		     n++;
		     m += matrix->val[i][i];
		   }
		}
		m /= (double)n;

		range = m - matrix->min_val;
		for (i = 0 ; i < matrix->size ; i++) {
			for (j = 0 ; j < matrix->size ; j++) {
						/* ratio: 1 identity, 0 non identity */
			  ratio = (float)(MIN2((double)(matrix->val[i][j]),m) - matrix->min_val)/range;
						/* to dist: MAT_MINVAL identity, MAT_MAXVAL non identity */
			  matrix->val[i][j] = MAT_MINVAL + (1.0 - ratio)*(double)(MAT_MAXVAL - MAT_MINVAL);
			}
		}
				/* raz diagonale			*/
		for (i = 0 ; i < matrix->size ; i++)
		   matrix->val[i][i] = MAT_MINVAL;

		matrix->min_val = MAT_MINVAL;
		matrix->max_val = MAT_MAXVAL;
	   }
	   else if (convert_method == DISTANCE_METHOD) {
				/* calcul par les distances entre a.a.	*/

				/* distance(i,j) is Sqrt[Sum(((i,k)-(k,j))^2) for all k	*/
				/* calcul des distances entre a.a.	*/
				/* on stocke dans la matrice inf	*/
				/* on garde la matrice sup		*/
				/* on ne touche pas a la diagonale	*/

		for (i = 0 ; i < matrix->size ; i++) {
			for (j = i+1 ; j < matrix->size ; j++) {
				dist = 0.0L;
				for (k = 0; k < matrix->size ; k++)
				   if (k >= i)		/* [i][k] et [j][k] matrice sup	*/
				      dist += (matrix->val[i][k]-matrix->val[j][k])*
						(matrix->val[i][k]-matrix->val[j][k]);
				   else		/* [k][i] et [k][j] matrice inf	*/
				      dist += (matrix->val[k][i]-matrix->val[k][j])*
						(matrix->val[k][i]-matrix->val[k][j]);
				      

						/* on remplit la matrice superieure	*/
				matrix->val[j][i] = (int)sqrt(dist);
				if (matrix->val[j][i] > matrix->max_val)
				   matrix->max_val = matrix->val[j][i];
				if (matrix->val[j][i] < matrix->min_val)
				   matrix->min_val = matrix->val[j][i];
			}
		}
	
				/* normalisation [MAT_MINVAL - MAT_MAXVAL] (0-100)	*/

		for (i = 0 ; i < matrix->size ; i++)
			matrix->val[i][i] = MAT_MINVAL;

		for (i = 0 ; i < matrix->size ; i++) {
		   for (j = i+1 ; j < matrix->size ; j++) {
		      matrix->val[i][j] = matrix->val[j][i] = MAT_MINVAL +
			(int)((double)(MAT_MAXVAL - MAT_MINVAL)*matrix->val[j][i]/(double)(matrix->max_val));
		   }
		}
		for (i = 0 ; i < matrix->size ; i++) {
		   for (j = 0; j < matrix->size ; j++) {
		      if (matrix->val[i][j] > matrix->max_val)
			 matrix->max_val = matrix->val[j][i];
		      if (matrix->val[i][j] < matrix->min_val)
			 matrix->min_val = matrix->val[j][i];
		   }
		}
	   }
	   else {
	      fprintf(stderr, "simple_convert: no proper conversion indicated\n");
	   }
	}
	else if (matrix->type == MAT_PAM_TYPE && totype == MAT_PROX_TYPE) {
		range = matrix->max_val - matrix->min_val;
		for (i = 0 ; i < matrix->size ; i++) {
			for (j = 0 ; j < matrix->size ; j++) {
						/* ratio: 1 identity, 0 non identity */
			  ratio = (double)(matrix->val[i][j] - matrix->min_val)/range;
						/* to prox: MAT_MAXVAL identity, MAT_MINVAL non identity */
			  matrix->val[i][j] = MAT_MINVAL + (int)(ratio*(double)(MAT_MAXVAL - MAT_MINVAL));
			}
		}
		matrix->min_val = MAT_MINVAL;
		matrix->max_val = MAT_MAXVAL;
	}
	else if (matrix->type == MAT_PROX_TYPE && totype == MAT_PAM_TYPE) {
					/* on place le 0 au milieu des valeurs	!!!	*/
					/* et on normalise avec le max et le min	*/
					/* d'une BLOSUM typique (62), cad 		*/
					/* MAT_BLOSUM_MAXVAL et MAT_BLOSUM_MINVAL, a	*/
					/* priori -4, 7					*/
					/* tous les termes diagonaux seront egaux	*/
					/* mais on ne peut pas inventer !!!		*/
		range = matrix->max_val - matrix->min_val;
		for (i = 0 ; i < matrix->size ; i++) {
			for (j = 0 ; j < matrix->size ; j++) {
						/* ratio: 1 identity, 0 non identity */
			  ratio = (double)(matrix->val[i][j] - matrix->min_val)/(double)range;
						/* to PAM: MAT_MAXVAL is identity,	*/
						/* MAT_MINVAL is non identity		*/
			  matrix->val[i][j] = MAT_PAM_MINVAL + (int)(ratio*(double)(MAT_PAM_MAXVAL - MAT_PAM_MINVAL));
			}
		}
		matrix->min_val = MAT_MINVAL;
		matrix->max_val = MAT_MAXVAL;
	}
	else if (matrix->type == MAT_DIST_TYPE && totype == MAT_PAM_TYPE) {
		range = matrix->max_val - matrix->min_val;
		for (i = 0 ; i < matrix->size ; i++) {
			for (j = 0 ; j < matrix->size ; j++) {
						/* ratio: 0 identity, 1 non identity */
			  ratio = (float)(matrix->val[i][j] - matrix->min_val)/range;
						/* to PAM: MAT_MAXVAL identity, MAT_MINVAL non identity */
			  matrix->val[i][j] = MAT_MINVAL + ratio*(MAT_MAXVAL - MAT_MINVAL);
			}
		}
		matrix->min_val = MAT_MINVAL;
		matrix->max_val = MAT_MAXVAL;
	}


	matrix->type = totype;

	if (verbose)
		printinfo_matrix(stderr, matrix, "MAT", vrai, faux);

	return vrai;
}

/* -------------------------------------------- */
/* @Function Boolean nodiag_convert()			*/
/* same as above but without diags adjustment	*/
/* added Dec 92 simple_convert was kept as is	*/
/* for compatibility...							*/ 
/* -------------------------------------------- */
Boolean nodiag_convert(matrix, totype)
	pam_matrix	*matrix;
	int			totype;
{
	int 	i, j;
	Boolean	substract;

						/* setup conversion parameters	*/		
	switch (totype) {
				
		case MAT_PROX_TYPE :
			substract = (matrix->type == MAT_DIST_TYPE);
			break;
		
		case MAT_DIST_TYPE :
			substract = (matrix->type != MAT_DIST_TYPE);
			break;
			
		case MAT_PAM_TYPE :
		default :
			return faux;
	}

						/* go on with conversion		*/
	if (substract) {
		for (i = 0 ; i < matrix->size ; i++)
			for (j = 0 ; j < matrix->size ; j++)
				matrix->val[i][j] = MAT_MAXVAL - matrix->val[i][j];
	}
	
	matrix->type = totype;
	
	return vrai;
}

/* -------------------------------------------- */
/* @Function Boolean chi2_convert()		*/
/* conversion to a distance matrix 		*/
/* (chi2 distance)				*/
/* -------------------------------------------- */

Boolean chi2_convert(matrix)
	pam_matrix	*matrix;
{
	int		i, j, k, taille;
	double	somme, val, kideu, max;
	double 	proba[MAT_SIZE_MAX][MAT_SIZE_MAX], 
			dist[MAT_SIZE_MAX][MAT_SIZE_MAX], marg[MAT_SIZE_MAX];


						/* must be symetrical		*/
	if (! sym_verify(matrix))
		return faux;
						/* must start with coocurences	*/
						/* ie smthing like a proximity	*/
						/* matrix			*/
	simple_convert(matrix, MAT_PROX_TYPE, 0, DISTANCE_METHOD);
	
						/* transfert 			*/
	taille = matrix->size;

	somme = 0.;
	for (i = 0; i < taille ; i++) 
	for (j = 0; j < taille ; j++) {
		proba[i][j] = (double)matrix->val[i][j];
		if (i >= j) somme += proba[i][j];
	}
						/* null summed matrix		*/
	if (somme == 0.)
		return faux;

						/* probas 						*/
	for (i = 0; i < taille ; i++)
	for (j = 0; j < taille ; j++)
		proba[i][j] /= somme;

						/* probas marginales 			*/
	for (i = 0; i < taille ; i++) {
		somme = 0.;
		for (k = 0; k < taille ; k++)
			somme += proba[i][k];
		if (somme == 0.)					/* ligne/col nulle				*/
			return faux;
		marg[i] = somme;
	}

						/* distance	du chi2 			*/	
	max = 0.;
	for (i = 0; i < taille ; i++)
	for (j = 0; j < taille ; j++) {
		kideu = 0.;
		for (k = 0; k < taille ; k++) {
			val = (proba[i][k]/marg[i]) - (proba[j][k]/marg[j]); 
			kideu += ((val*val)/marg[k]);
		}
		val = sqrt(kideu);
		dist[i][j] = val;
		if (val > max) 
			max = val;
	}

						/* transfert		 			*/
	 
	for (i = 0; i < taille ; i++)
	for (j = 0; j < taille ; j++)
		matrix->val[i][j] = (int)(dist[i][j]*MAT_MAXVAL/max);

	matrix->type = MAT_DIST_TYPE;
	
	return vrai;
}

/* -------------------------------------------------------------------- */
int get_sMatrixMaxVal(void)
{
	return sMatrixMaxVal;
}

/* -------------------------------------------------------------------- */
int get_sMatrixMinVal(void)
{
	return sMatrixMinVal;
}

/* -------------------------------------------------------------------- */
float get_sIndelOpeRatio(void)
{
   return sIndelOpeRatio;
}

/* -------------------------------------------------------------------- */
void set_sIndelOpeRatio(float value)
{
   sIndelOpeRatio = value;
   
   if (sIndelOpeRatio < sIndelGapGapRatio)
      sIndelGapGapRatio = sIndelOpeRatio;

   return;
}

/* -------------------------------------------------------------------- */
float get_sIndelExtRatio(void)
{
   return sIndelExtRatio;
}

/* -------------------------------------------------------------------- */
void set_sIndelExtRatio(float value)
{
   sIndelExtRatio = value;
   
   return;
}

/* -------------------------------------------------------------------- */
float get_sIndelGapGapRatio(void)
{
   return sIndelGapGapRatio;
}

/* -------------------------------------------------------------------- */
void set_sIndelGapGapRatio(float value)
{
   sIndelGapGapRatio = value;
   
   if (sIndelOpeRatio < sIndelGapGapRatio)
      sIndelOpeRatio = sIndelGapGapRatio;

   return;
}

/* ------------------------------------------------------------	*/
/* repartit val sur les symboles lies a pos (dans tab) selon	*/
/* la matrice de substitution mat (en fait ses freq calculees	*/
/* par la fonction compute_freq_matrix				*/
/* ------------------------------------------------------------	*/
double matcount(double *tab, int pos, int len, pam_matrix *mat, double val)
{
   int		isymb;
   double	d = 0.0L;

			/* if not an amino-acid i.e. BJOUXZ	*/
   if (sProbaSwissProt[pos] <= 0.0L)
      return 0.0L;

   for (isymb = 0;isymb < len; isymb++) {
      if (sProbaSwissProt[isymb] > 0.0L) {
			/* do not take into account BJOUXZ	*/
	 tab[isymb] += (val*mat->freq[pos][isymb]);
         d += (val*mat->freq[pos][isymb]);
      }
   }

   return d;

}

/* ------------------------------------------------------------	*/
/* compute the freq Qij from BLOSUM matrix values		*/
/* ------------------------------------------------------------	*/
void compute_freq_matrix(pam_matrix *mat, int verbose)
{
   int	isymb,jsymb,nsymb;
   double pbtot[26];

                        /* calcule une frequence sur les symboles lies	*/
                        /* dans la matrice mat				*/


   nsymb = strlen(mat->vocab);

   for (isymb = 0;isymb < nsymb; isymb++) {
      pbtot[isymb] = 0.0L;
      if (sProbaSwissProt[isymb] <= 0.0L) {
	for (jsymb = 0; jsymb< nsymb; jsymb++)
	   mat->freq[isymb][jsymb] = 0.0L;
      }
      else {
         for (jsymb = 0; jsymb< nsymb; jsymb++) {
			/* BLOSUM Mij is 2*log2(Pij/PiPj)       */
			/* so Pij/PiPj=2^(Mij/2)                */
            if (sProbaSwissProt[jsymb] > 0.0L) {
	       mat->freq[isymb][jsymb] = sProbaSwissProt[jsymb]*pow(2.0L, mat->val[isymb][jsymb]/2.0L);
	    }
	    else {
	       mat->freq[isymb][jsymb] = 0.0L;
            }
	    if (verbose)
	       fprintf(stderr,"%c %c %f\n",
		  mat->vocab[isymb],mat->vocab[jsymb],mat->freq[isymb][jsymb]);
	    pbtot[isymb] += mat->freq[isymb][jsymb];
         }
      }
      if (verbose)
         fprintf(stderr,"--> Sum %c %f (proba simple %f)\n",
	    mat->vocab[isymb],pbtot[isymb],sProbaSwissProt[isymb]);
   }

   return;

}

