#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <time.h>             /* pour memset */
#include <unistd.h>		/* pour getopt	*/



#include "swelfe.h"
#include "swalign.h"
#include "lecture.h"
#include "macros.h"
#include "util.h"
#include "minimier.h"
#include "stats.h"
#include "sortie.h"
#include "libabiutils.h"
#include "Nmatrix.h"
#include "stats_sequence.h"
#include "fonctions_main.h"

/*==================== PROTOTYPES FONCTIONS LOCALES =================================*/


/*================================  FONCTIONS =================================*/



/* ==================== affichage des options =====================*/

void PrintResumer (tyParametres	parms, int SPN, int proba, char *nomFi, char *matname,
            float (*gscore)(int s1, int s2, void *matthresh),
            int (*Igscore)(int s1, int s2, void *matthresh) ){

  fprintf(stdout, " @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@\n");
  fprintf(stdout, " @@@@@@@@@@  Parameters used  :    @@@@@@@@@@\n");
  fprintf(stdout, " @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@\n\n");

  fprintf(stdout, " @@ Type of file :  ");
    switch(SPN) {
		      case 0:   fprintf(stdout, " 3D \n");    break;
		      case 1:   fprintf(stdout, " amino acid sequences\n");   break;
		      case 2:   fprintf(stdout, " DNA sequences \n");   break;
		   }
 
  fprintf(stdout, " @@ File : %s  format ", parms.st[0]);
    switch(parms.format[0]) {
		      case PDB_FORMAT:   
		        if (SPN == STRUCTURE)
        		      fprintf(stdout, " PDB \n"); 
        		else 
        		      fprintf(stdout, "  fasta \n");         		    
        		break;
		      case BIN_FORMAT:   fprintf(stdout, " binary\n");   break;
		      case ASC_FORMAT:   fprintf(stdout, " alpha \n");   break;
		   }
#ifdef ADVANCED
  fprintf(stdout, " @@ 2nd file : %s  format ", parms.st[1]);
    switch(parms.format[1]) {
		      case PDB_FORMAT:   
		        if (SPN == STRUCTURE)
        		      fprintf(stdout, " PDB \n"); 
        		else 
        		      fprintf(stdout, "  fasta \n");         		    
        		break;
		      case BIN_FORMAT:   fprintf(stdout, " binary\n");   break;
		      case ASC_FORMAT:   fprintf(stdout, " alpha \n");   break;
		   }
#endif

  fprintf(stdout, " @@ Number of alignments to show : %3d \n", parms.nbest);
  fprintf(stdout, " @@ Minimum score  %5.2f \n", parms.minscore);
  fprintf(stdout, " @@ Gap penalty : opening : %3.2f + k * extension : %3.2f (k total gap length)\n", parms.gap_o, parms.gap_e);
  fprintf(stdout, " @@ Overlap allowed : %2.3f \n", parms.recover);
  if (parms.sim == 1)
	fprintf(stdout, " @@ Algorithm : Huang-Miller\n");
  else
	fprintf(stdout, " @@ Algorithm : Smith-Waterman\n");

    switch(SPN) {
		      case 0:  		  
		              fprintf(stdout, " @@ maximum RRMSD  %3.2f \n", parms.rrmsd); 
		              fprintf(stdout, " @@ Threshold for the score %5.2f      \n", parms.threshold); 
		              fprintf(stdout, " @@ Minimum length for repeats : %d \n", parms.length); 
	              
#ifdef ADVANCED                      
		              fprintf(stdout, " @@ Number of alpha classes : %d \n", parms.angleMax); 
  		              fprintf(stdout, " @@ Number of structures conserved for the algorithm %d \n", parms.nbkept); 
		              fprintf(stdout, " @@ Maximum value for cross RMS : %5.2f \n", parms.cross_rms); 
                      fprintf(stdout, " @@ Number of structures for Gok %d \n", parms.nbStructSet);
                      fprintf(stdout, " @@ Number of stuctures for pymol %d \n", parms.nbStructpymol);
                     fprintf(stdout, " @@ Output file : %s \n", nomFi );
#endif		          
		          break;
		      default:  
		        fprintf(stdout, " @@ P-value %5.2f\n", 1- parms.proba);
                      fprintf(stdout, " @@ Declumping method : %d \n", proba);
                      if (SPN == PROTEIQUE)
    		      		 fprintf(stdout, " @@ Substitution matrix %s \n", matname);
	                 fprintf(stdout, " @@ Number of random sequences : %5d  \n", parms.nb_shuffle);
		          break;  
		    }
    
    fprintf(stdout, "\n @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@\n\n\n");

 }



/* --------------------------------------------	*/
/* Affichage de l'aide 				*/
/* --------------------------------------------	*/

void PrintHelp(char *prog)
{
#ifdef ADVANCED 
  fprintf(stderr, "%s: Usage: -[a|b|s] 1st_input_file [-[A|B|S] 2nd_input_file] -P [0|1|2] options \n",prog);
#else
  fprintf(stderr, "%s: Usage: -s input_file -P [0|1|2] options \n",prog);
#endif
   fprintf(stderr, "  ********************        FILES                  ********************************** \n");
#ifdef ADVANCED 
  fprintf(stderr, "  -a 1st_file : first file is in alpha format (output from phipsi)\n");
  fprintf(stderr, "  -b 1st_file : first file is in binary format (output from phipsi or fasta sequences in binary format\n");
#endif
    fprintf(stderr, "  -s file : file is in PDB format or fasta format for sequences\n");
    fprintf(stderr, "          (PDB file can be compressed in .Z or .gz format)\n");
#ifdef ADVANCED 
  fprintf(stderr, "  -A 2nd_file : second file is in alpha format (output from phipsi)\n");
  fprintf(stderr, "  -B 2nd_file : second file is in binary format (output from phipsi or fasta sequences in binary format)\n");
  fprintf(stderr, "  -S 2nd_file : second file is in PDB format or fasta format for sequences\n");
    fprintf(stderr, "          (PDB file can be compressed in .Z or .gz format)\n");
  fprintf(stderr, "  If 2nd_file is not given, \"auto\" SW alignment is achieved on the structure(s) of the first file\n");
#endif
  fprintf(stderr, "  ********************        STRUCTURE / SEQUCENCE  ********************************** \n");
  fprintf(stderr, "  -P<0/1/2> 0 : structure analysis\n");
  fprintf(stderr, "  -P<0/1/2> 1 : proteique sequence analysis\n");
  fprintf(stderr, "  -P<0/1/2> 2 : nucleique sequence analysis \n");
  fprintf(stderr, "  ********************        OPTIONS              ********************************** \n");
  fprintf(stderr, "  -l  output on one line per match (several lines output by default) \n");
  fprintf(stderr, "  -n <number> number of best subalignments to show (even if they are not\n");
  fprintf(stderr, "     statistically significant for sequences) \n");
  fprintf(stderr, "     - default for structure: %d\n", DEFAULT_NBEST);
  fprintf(stderr, "     - default for sequence: -1, i.e. the algorithm rely on statistical criteria for stopping output)\n");
  fprintf(stderr, "  -L <number> minimal length for the repeat %d \n", DEFAULT_LENGTH);
  fprintf(stderr, "  -m <minscore> : minimum score for output subalignment\n");
  fprintf(stderr, "  -z <default minscore> : use default minimum score for output subalignment(def structure %.0f protein %.0f  nucleic %.0f)\n", DEFAULT_MINSCORE_STRUCT, DEFAULT_MINSCORE_PROT, DEFAULT_MINSCORE_NUC);
  fprintf(stderr, "  -N <proba> : probability for dna and protein (def:%f)\n", 1-PROBA);
  fprintf(stderr, "  gaps :  \n"); 
  fprintf(stderr, "  -G -1 : default value for gap open and extension penalty :  %5.2f structure, %5.2f protein, %5.2f nucleic)\n", DEFAULT_GAP_STRUCT, DEFAULT_GAP_PROT, DEFAULT_GAP_NUC);
  fprintf(stderr, "          default gap extention penalty  : %5.2f structure, %5.2f protein, %5.2f nucleic)\n", DEFAULT_GAP_STRUCT_EXT, DEFAULT_GAP_PROT_EXT, DEFAULT_GAP_NUC_EXT);
  fprintf(stderr, "          gap cost = gap_open + k * gap_ext (k being total gap length) \n");
  fprintf(stderr, "  -G <gap> : other value for gap open penalty \n");
  fprintf(stderr, "  -g <gap> : other value for gap extension penalty \n");
  fprintf(stderr, "  -r <float> : ratio of tolerated recovering [0,1] (def:%4.2f)\n", DEFAULT_RECOVER);
  fprintf(stderr, "  -q : output matches are sorted by scores (default: sort is done on RMS)\n");
  fprintf(stderr, "  -i : standard output \n");
  fprintf(stderr, "  -j : error output \n");
  fprintf(stderr, "  -W : Smith/Waterman algorithm is to be used (default: Huang/Miller algorithm)\n");
  fprintf(stderr, "  -F : No sequence/structure size limit, (default:%d when SW is used)\n", NB_MAX_RESIDUS);
  fprintf(stderr, "  ********************        OPTIONS   FOR SEQUENCES ONLY        ********************************** \n");
  fprintf(stderr, "  -x matrix (if amino acid or DNA sequence) \n");
  fprintf(stderr, "  -D : number of shuffled sequences used to calculate the statistical significance of the scores, default : %d\n", NB_SHUFFLE);
  fprintf(stderr, "  -p second method of Waterman and Vingron for statistical significance : declumping estimation \n");
#ifdef ADVANCED 
  fprintf(stderr, "  -R if option -p : number of repeats by shuffled sequences to calculate the statistical significance \n");
#endif

  fprintf(stderr, "  ********************        OPTIONS   FOR STRUCTURES ONLY           ********************************** \n");
  fprintf(stderr, "  -M <minrms> : minimum rms for output subalignment(def:%f)\n", DEFAULT_MINRMS);
  fprintf(stderr, "  -u <normalize minrms> : normalize minimum rms for output subalignment (Betancourt, Skolnick, 2001) (def:%f)\n", DEFAULT_RRMSD);
  fprintf(stderr, "  -U use RMSD instead of Relative RMSD \n");
#ifdef ADVANCED 
  fprintf(stderr, "  -c <number_of_alpha_classes> (def:%d)\n", DEFAULT_ANGLE_MAX);
  fprintf(stderr, "  -k <integer> : number of wanted output structures (def:%d)\n", DEFAULT_KEPT_STRUCTURES);
  fprintf(stderr, "  -y : score struct : threashold - abs(a1-a2), (by default )    \n");
#endif
  fprintf(stderr, "  -w <threshold> : value of threshold for angle score (def:%10.2f)\n", DEFAULT_THRESHOLD );
  fprintf(stderr, "     note: score is detailed in paper\n");
#ifdef ADVANCED 
  fprintf(stderr, "  -Z <rms_threshold> : value of threshold for crossed RMS of two fragments to be considered as dimer like (def:%10.2f)\n", DEFAULT_CROSSEDRMS);
#endif 
  fprintf(stderr, "  -e : to extend alignment for amino acid distant than less than x Angstrom \n");

#ifdef ADVANCED 
  fprintf(stderr, "  ********************        OPTIONAL OUTPUT         ********************************** \n");
   fprintf(stderr, "  -t <number> : number of structure for gok output file \n");
   fprintf(stderr, "  -T <number> : number of structure for pymol output file \n");
   fprintf(stderr, "  -o <output file>  : only for four options above - an extention will be added for gok(%s), molscript (%s) --- and only for 1 protein against 1 protein \n",  EXTENSION_GOK,  EXTENSION_PYMOL);    
#endif
  return;
}


int egal (float a, float b){ /*compare 2 float*/
    if (ABS(a-b) < EPSILON)
        return 1;
    return 0;
}


#if 0
/* ------------------------------------	*/
 void sPrintAngles(tyProt *tp, int start, int wlen, int bizarre)
{
  int j;

  /* print group angles values		*/
  for (j=start;j<start+wlen;j++) {
    if(bizarre == 1){
      if(tp->tAnglesRecodes[j] < 0 || tp->tAnglesRecodes[j] >= DEFAULT_ANGLE_MAX )
	fprintf(stderr,"%4d|",tp->tAnglesRecodes[j]);}
    else  
      fprintf(stderr,"%4d|",tp->tAnglesRecodes[j]);
  }
  fprintf(stderr,"\n");
  
  return;
}

#endif
/* ----------------------------------------------------------- */
 void PrintArbreScan(FILE *fout, tyProt *p0, int proba, int SPN,float *mat_or_thresh, float gap_o, float gap_e, int (*qscompar)(const void *, const void *))
{
  int i;  
  SWResult *swres;

  i=0;
  swres=get_minimier_swres(i, qscompar);
  if(swres==NULL){
    fprintf(stderr, "PrintArbreScan:: No protein returned from the minimier\n");
    return;
  }
  while(swres !=NULL) {
    PrintResult2(fout, swres, p0, proba, SPN,0.0,0.0, mat_or_thresh,  gap_o,  gap_e);
    i++;
    swres=get_minimier_swres(i, qscompar);
  }

  return;
}

#if 0

/* ----------------------------------------------------------- */
 void FreeSWResultInArbre(int (*qscompar)(void *, void *))
{
  int i;  
  SWResult *swres;

  i=0;
  swres=get_minimier_swres(i, qscompar);
  
  while(swres !=NULL) {
    /* debug */
    fprintf(stderr, "freeing name %s\n",swres->prot1->nomProt); 
    FreeSWResult(swres, FREEPROT);
    i++;
    swres=get_minimier_swres(i, compar);
  }

  return;
}

#endif
