/*
 * getSSR.h
 */

/* -------------------------------------------
 * Constantes de dimensionnement
 */

#define MAXSIZ 1000
#define EOS     ((char) 0xff)   /* end of encoded sequence */

#define MAX_N_PM	100000000	/* ~ max N for PM ran. gen. */
#define MAX_SYMB    32  /* 'A' -> 'Z' */
#define ALPHA_DNA   "ACGT"
#define ALPHA_SIZ   4

#define BIT_SHIFT   5	/* log2(MAX_SYMB) - # bits needed   */
#define MAX_KSIZE   6	/* sizeof(Int32) * 8 / BIT_SHIFT    */

#define MAX_PITER   10000 /* max # of pval iterations	    */

/* -------------------------------------------
 * Structures de donnees
 */

typedef struct {    /* alphabet + hash table */
    int	    alphasize;	    /* size of alphabet		    */
    int	    bitshift;	    /* log2(alphasize)		    */
    int	    wsize;	    /* 2^bitshift		    */
    int	    ksize;	    /* word size		    */
    int   tsize;	    /* (2^bitshift)^(ksize-1)	    */
    int   smask;	    /* symbol mask useful bits	    */
    int   wmask;	    /* word mask useful bits	    */
    int   tsum;	    /* sum of all hashtable counts  */
    int   **table;	    /* hashtable		    */
    char    alphabet[MAX_SYMB+1];
} HashTable;
   
typedef struct {    /* motif elements table */
    int	    large;	    /* larger shift (j) */
    int	    small;	    /* smaller shift (i) */
	int 	includ;		/* 0 means that it is not to be included*/
} MotifsTable;

/* -------------------------------------------
 * Macros standards
 */

#ifndef NEW
#define NEW(typ)		(typ*)malloc(sizeof(typ)) 
#define NEWN(typ, dim) 		(typ*)malloc((unsigned long)(dim) * sizeof(typ))
#define REALLOC(typ, ptr, dim)	(typ*)realloc((void *) (ptr), (unsigned long)(dim) * sizeof(typ))
#define FREE(ptr)		free(ptr)
#endif

#ifndef MIN
#define MIN(x, y) (((x) < (y)) ? (x) : (y))
#endif

#ifndef MAX
#define MAX(x, y) (((x) > (y)) ? (x) : (y))
#endif

#ifndef ABS
#define ABS(x) (((x) >= 0) ? (x) : -(x))
#endif

#if PROTO
# define        P(s) s
#else
# define 	P(s) ()
#endif

typedef void	(*SeedFunc) P(( long seed ));
typedef	float	(*RanFunc)  P(( void ));


/* -------------------------------------------
 * Prototypes
 */
				     /* gshuf_ran.c */
void 	sranPM(long seed);
float 	ranPM(  );
void 	sranLE( long seed );
float 	ranLE(  );
void 	sranKN( long seed );
float 	ranKN( );
void 	sranUN( long seed );
float 	ranUN(  );
