/* ---------------------------------------------------------------- */
/* Copyright (c) Atelier de BioInformatique			    */
/* @file: libnr.h						    */
/* @desc: nr library / include file				    */
/*								    */
/* @history:							    */
/* @+	    <Gloup> : Tue Oct 29 1996 : Created			    */
/* ---------------------------------------------------------------- */

#ifndef _H_libnr

#define _H_libnr

/* ==================================================== */
/* constantes						*/
/* ==================================================== */

#define kMaxExpoList    1000         /* maximum exponent in a product list */

/* ==================================================== */
/* structures de donnees				*/
/* ==================================================== */

typedef struct {
        int     *power;     /* power[i] is the exponent of i */
        int     n;          /* number of terms in power list */
} PLIST;

/* ==================================================== */
/* macros						*/
/* ==================================================== */

#define nr_prod_get(list,i)	(list->power[i])
#define nr_prod_set(list,i,p)	(list->power[i] = (p))
#define nr_prod_n(list)		(list->n)
#define nr_prod_pow(list,i,p)	nr_prod_set (list, (i), nr_prod_get (list, (i)) + (p))
#define nr_prod_mult(list,i)	nr_prod_pow (list, i, 1)
#define nr_prod_div(list,i)	nr_prod_pow (list, i, (-1))

/* ==================================================== */
/* prototypes						*/
/* ==================================================== */

			/* nr_util.c			*/
			
void	nr_error    (char *error_text);
float	*nr_vector  (int nl, int nh);
int	*nr_ivector (int nl, int nh);
double	*nr_dvector (int nl, int nh);
float	**nr_matrix (int nrl, int nrh, int ncl, int nch);
double	**nr_dmatrix (int nrl, int nrh, int ncl, int nch);
int	**nr_imatrix (int nrl, int nrh, int ncl, int nch);
float	**nr_submatrix (float **a, int oldrl, int oldrh, int oldcl,
			int oldch, int newrl, int newcl);
void	nr_free_vector	(float *v, int nl, int nh);
void	nr_free_ivector (int *v, int nl, int nh);
void	nr_free_dvector (double *v, int nl, int nh);
void	nr_free_matrix	(float **m, int nrl, int nrh, int ncl, int nch);
void	nr_free_dmatrix (double **m, int nrl, int nrh, int ncl, int nch);
void	nr_free_imatrix (int **m, int nrl, int nrh, int ncl, int nch);
void	nr_free_submatrix   (float **b, int nrl, int nrh, int ncl, int nch);
float	**nr_convert_matrix (float *a, int nrl, int nrh, int ncl, int nch);
void	nr_free_convert_matrix (float **b, int nrl, int nrh, int ncl, int nch);

			/* nr_primes.c			*/
double	nr_logprime	(int p);
void	nr_dofactor	(int *small, int *large, int i, int n);
PLIST	*nr_prod_new	(int n);
void	nr_prod_rel	(PLIST *list);
void	nr_prod_init	(PLIST *list);
void	nr_prod_fact	(PLIST *list, int fact, int sign);
double	nr_prod_compute (PLIST *list);

			/* nr_cntg.c			*/
void	nr_cntab1 (int **nn, int ni, int nj, float *chisq, float *df, 
		   float *prob, float *cramrv, float *ccc);
void	nr_cntab2 (int **nn, int ni, int nj, float *h, float *hx,
		   float *hy, float *hygx, float *hxgy, float *uygx,
		   float *uxgy, float *uxy);

			/* nr_fisher.c			*/
double	nr_fisher2x2  (int A, int B, int C, int D);
int	nr_fishtail   (int A, int B, int C, int D);

			/* nr_gamma.c			*/
double	nr_gammln   (double xx);
void	nr_gser	    (double *gamser, double a, double x, double *gln);
void	nr_gcf	    (double *gammcf, double a, double x, double *gln);
double	nr_gammq    (double a, double x);

			/* nr_random.c			*/
void	nr_sranPM   (long seed);
float	nr_ranPM    (void);
void	nr_sranLE   (long seed);
float	nr_ranLE    (void);
void	nr_sranKN   (long seed);
float	nr_ranKN    (void);
void	nr_sranUN   (long seed);
float	nr_ranUN    (void);

#endif
