/* score_struct.c ABI:  novembre 2005	*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>


#include "swelfe.h"
#include "swalign.h"
#include "score_struct.h"
#if 0
#include "angles.h"
#endif

#define CSTE_M  20.0
#define CSTE_D0 5.0

#if 0
static int calcule_nbgaps(char *ali){

    int nb = 0, i;

    for(i=0; i<(int)strlen(ali); i++){
        if(ali[i] != 's')
            nb ++;
        }
    return nb;
}
#endif

static double levitt_sim(COORDS *s1, COORDS *s2, float d0, DtMatrix4x4 matrix4x, int lenmatch)
/* ------------------------------------------------------------------------ */
/* pour calculer le score de gerstein et levitt */
{
  COORDS p2;
  double dist, sim, d;
  int   i;

  sim = 0.0L;
  for (i = 0; i < lenmatch; i++) {
    dist = 0.0L;
    p2 = crdRotate(s2[i],matrix4x);
    d =  s1[i].x - p2.x;
    dist += d * d;
    d =  s1[i].y - p2.y;
    dist += d * d;
    d =  s1[i].z - p2.z;
    dist += d * d;
    sim += 1.0L/(1.0L+dist/(d0*d0));  /*dist est deja au carre*/
  }
   /* fprintf(stderr," sim %f\n",sim);*/

  return sim;
}

/*score gerstein levitt */
double calcule_score_struct( COORDS *pp0, COORDS *pp1, int len_ali, int len_match, DtMatrix4x4 matrix4x)
{

    int nb_gaps;
    double resultat;
    double score;

    /* OLD jompo: nb_gaps = calcule_nbgaps (sali); */
    nb_gaps = len_ali-len_match;

    /* fprintf(stderr, "nb gaps : %d len ali %d\n", nb_gaps, (int)strlen(sali)); */

    resultat = levitt_sim(pp0, pp1, CSTE_D0, matrix4x, len_match);
        /*formule : 
        Score = M * (somme  (    1      ) - Ngaps )
                             _________      _____
                             1 + (dij/do)2    2
        resultat calcule la somme
        */
        


    score = CSTE_M * ( resultat - nb_gaps / 2);
 /*pb vrifications des gaps en bout de chaine ???*/
/*  fprintf(stderr," *******************score %f, nb_gaps %d\n",score, nb_gaps);*/

    return score;
}


/*    calcul du TM score  */
double calcule_tm_score( COORDS *pp0, COORDS *pp1, int lenmatch, tyProt *p0, DtMatrix4x4 matrix4x)
{

    double resultat;
    double score;
    double Do;
    
    
    /*on prend longueur struct /2       */
    
        Do = 1.24 * pow((((float)(p0->nbRes)/2.0) -15), (float)(1.0/3.0))-1.8;

        resultat = levitt_sim( pp0,pp1, Do,  matrix4x, lenmatch);
        /*formule : 
        
        Score = 1 * (somme  (    1      ))
               ----           _________    
                Ln             1 + (dij/do)2   
        resultat calcule la somme
        
        
        */
        
/*   fprintf(stderr, "res : %f Do : %f\n", resultat, Do);*/
 /* fprintf(stderr," *****res  %f\n",resultat);*/


        score = (2.0/(double)(p0->nbRes)) * ( resultat);
  /*         fprintf(stderr, "res : %f Do : %f score %f\n", resultat, Do, score);*/

 /*pb vrifications des gaps en bout de chaine ???*/
/*  fprintf(stderr," *******************score %f, nb_gaps %d\n",score, nb_gaps);*/

    return score;
}


#if 0 /*tout commente en dessous !! */

/*score / freq angles */
void calcule_score_freq (SWResult *swres, tyProt *p0, float *mat_or_thresh, float gap_o, float gap_e) {


    int i,j, longueur, position_0, position_1;
    double angle0, angle1, freq;
    
    for(i=0; i<swres->nmatch; i++){
    	longueur = (int)strlen(swres->sali[i]);
        position_0 = position_1 = 0;    
        swres->score_freq[i] = 0;
    
    
        for(j=0; j<longueur; j++){
            angle0 = p0->tAngles[swres->posd0[i]+position_0];
            angle1 = swres->prot1->tAngles[swres->posd1[i]+position_1];
            freq = AnglesAlpha[180+(int)(angle0/DIVANGLE)] * AnglesAlpha[180+(int)(angle1/DIVANGLE)];
        
    	   if(swres->sali[i][j] == SUBST) {
                (swres->score_freq[i]) += (anglescore(angle0,angle1, mat_or_thresh))/(freq*1000);
                 
/*                 fprintf(stderr, "score %f %f %f\n", freq*1000, anglescore(angle0,angle1, mat_or_thresh), swres->score_freq[i]);
                 
  */               
                 
/*fprintf(stderr, "score bis %f \n", (anglescore(angle0,angle1, mat_or_thresh)/
                 (freq*1000)));
  */               
                 
                position_0 ++;
                position_1 ++;
            }
            else if (swres->sali[i][j] == INSER){
                (swres->score_freq[i]) += gap_o;
                position_0 ++;
                }
            else  {
                (swres->score_freq[i]) += gap_o;
                position_1 ++;
                }
        }

    }

}
/*score * freq angles */
void calcule_score_par_freq (SWResult *swres, tyProt *p0, float *mat_or_thresh, float gap_o, float gap_e) {


    int i,j, longueur, position_0, position_1;
    double angle0, angle1, freqA, freqB;
    double max_angle = 0.02371223;  /*angle maximum*/ 
    double min_angle = 0.0003717822;  /*angle min*/
    
    /*1ere formule : score * (1-(freqA - min) / (max - min)* (1-(freqB - min) / (max - min))
    et inverse si score negatif */
 
 /*2eme formule : score * 1-log freqA * 1-log freqB */
    
    for(i=0; i<swres->nmatch; i++){
    	longueur = (int)strlen(swres->sali[i]);
        position_0 = position_1 = 0;    
        swres->score_par_freq[i] = 0;
    
    
        for(j=0; j<longueur; j++){
            angle0 = p0->tAngles[swres->posd0[i]+position_0];
            angle1 = swres->prot1->tAngles[swres->posd1[i]+position_1];
            freqA = AnglesAlpha[180+(int)(angle0/DIVANGLE)] ;
            freqB = AnglesAlpha[180+(int)(angle1/DIVANGLE)];
        
    	   if(swres->sali[i][j] == SUBST) {
    	        /*(swres->score_par_freq[i]) += (anglescore(angle0,angle1, mat_or_thresh))* (1- freqA)* (1-freqB);*/

                /*(swres->score_par_freq[i]) += (anglescore(angle0,angle1, mat_or_thresh))* (1- log(freqA))* (1-log(freqB));     */
            (swres->score_par_freq[i]) += (anglescore(angle0,angle1, mat_or_thresh))* (1-(freqA-min_angle)/(max_angle - min_angle))* (1-(freqB-min_angle)/(max_angle - min_angle));
                
    	     /*  if (anglescore(angle0,angle1, mat_or_thresh)>0)
                (swres->score_par_freq[i]) += (anglescore(angle0,angle1, mat_or_thresh))* (1-(freqA-min_angle)/(max_angle - min_angle))* (1-(freqB-min_angle)/(max_angle - min_angle));
                else 
                (swres->score_par_freq[i]) += (anglescore(angle0,angle1, mat_or_thresh))* ((freqA-min_angle)/(max_angle - min_angle))* ((freqB-min_angle)/(max_angle - min_angle));  */
              /*  fprintf(stderr, "score %f \n", (anglescore(angle0,angle1, mat_or_thresh))* (1-(freqA-min_angle)/(max_angle - min_angle))* (1-(freqB-min_angle)/(max_angle - min_angle)));*/
                 
                 
                 
/*fprintf(stderr, "score bis %f \n", (anglescore(angle0,angle1, mat_or_thresh)/
                 (freq*1000)));
  */               
                 
                position_0 ++;
                position_1 ++;
            }
            else if (swres->sali[i][j] == INSER){
                (swres->score_par_freq[i]) += gap_o;
                position_0 ++;
                }
            else  {
                (swres->score_par_freq[i]) += gap_o;
                position_1 ++;
                }
        }
fprintf(stderr, "score bis %f \n", (swres->score_par_freq[i]) );
                 

    }

}


/*---------------------------------------------	    */
/* calcule le score de difference d'angle           */
/*---------------------------------------------	    */
void CalculScoreAngles(tyProt *p1, int posd1, 
		       tyProt *p2, int posd2, char *sali, float *score_angle)
{
   int		*ss1 = p1->tAngles + posd1,
   		*ss2 = p2->tAngles + posd2;
		              /*     fprintf(stderr," tmp debut  %f \n", *score_angle);*/

    float threshold = 30.0;
    *score_angle=0.0;
    
   while (*sali) {
     if(*sali == SUBST) {
       *score_angle += anglescore(*ss1, *ss2, &threshold);
                   /*fprintf(stderr," tmp %f \n", *score_angle);*/

       ss1++;
       ss2++;
     }
     else if (*sali == INSER) {
       *score_angle -= 200;
       ss1++;
     }
     else { /* delet */
       *score_angle -= 200;
       ss2++;
     }
     sali++;
   }
   return;
   
}

#endif

/*calcul du rms independant de la longueur du match, cf article tm score */
double calcule_rms_bis( COORDS *pp0, COORDS *pp1, int lenmatch, tyProt *p0,tyProt *p1,  double *rms)
{

double new_rms;
int n = lenmatch;
double c;
double giration1=0.0, giration2=0.0;
float mean_x =0.0, mean_y=0.0, mean_z=0.0; 
int i;

/*calcul du rayon de giration*/
  for (i=0;i<lenmatch;i++) {
    mean_x += pp0[i].x;
    mean_y += pp0[i].y;
    mean_z += pp0[i].z;
  }

    mean_x /= (double)(lenmatch);
    mean_y /= (double)(lenmatch);
    mean_z /= (double)(lenmatch);
  
  for (i=0;i<lenmatch;i++) {
giration1 += pow((pp0[i].x - mean_x), 2) + pow((pp0[i].y - mean_y), 2) + pow((pp0[i].z - mean_z), 2) ;
  }

giration1 /= lenmatch;
giration1 = sqrt(giration1);

 mean_x =0.0;
 mean_y=0.0;
 mean_z=0.0; 

  for (i=0;i<lenmatch;i++) {
    mean_x += pp1[i].x;
    mean_y += pp1[i].y;
    mean_z += pp1[i].z;
  }

    mean_x /= (double)(lenmatch);
    mean_y /= (double)(lenmatch);
    mean_z /= (double)(lenmatch);
  
  for (i=0;i<lenmatch;i++) {
giration2 += pow((pp1[i].x - mean_x), 2) + pow((pp1[i].y - mean_y), 2) + pow((pp1[i].z - mean_z), 2) ;
  }

giration2 /= lenmatch;
giration2 = sqrt(giration2);

c = 0.42 - 0.05 * (n-1)*exp(-1* (n-1) / 4.7) + 0.63 * exp(-1*(n-1) / 37.0);

new_rms = (*rms)/sqrt(pow(giration1, 2)+pow(giration2, 2) -2*c * giration1 *giration2 );

/*fprintf(stderr, "tototootoot %f %f %f %f %f %f %d \n", new_rms, c, giration1, giration2, *rms, sqrt(pow(giration1, 2)+pow(giration2, 2) -2*c * giration1 *giration2 ), n);*/

return new_rms;
}
