/* ---------------------------------------------------  */
/* verif_input.c : ABI 1st version April 2008                */
/* verify input for swelfe                              */
/* ---------------------------------------------------  */

/* ---------------------- HEADER A INCLURE  ----------- */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <time.h>             /* pour memset */
#include <unistd.h>     /* pour getopt  */


/* ---------------------- HEADER LOCAUX --------------- */
#include "swelfe.h"
#include "swalign.h"
#include "lecture.h"
#include "macros.h"
#include "util.h"
#include "zlib.h"
#include "libunz.h"

/*--------------------------------------------- */
/* main routine                 */
/*--------------------------------------------- */

int main(int argc, char *argv[])
{
   tyProt    *p0=NULL;
   FILE      *pFi= NULL, *fol = NULL;
   char	     ligne[512];
   char      *alphabet = "ABCDEFGHIJKLMNOPQRSTUVWXYZ";

   /* Selon structure, sequence proteique ou nucleique  */

       /* ------------------------------------ */
       /* Ouverture des fichiers d'entree  */
       /* ------------------------------------ */

       if (argc < 2) {
         fprintf(stdout, "Error: No input file !\n");
         exit(1);
       }

       if ((pFi=fopen(argv[1], "rt"))==NULL) {
          fprintf(stdout, "Error: cannot open %s\n", argv[1]);
          exit(1);
       }

       if (argv[1][strlen(argv[1])-1]=='Z' && argv[1][strlen(argv[1])-2] == '.') {
         if ((fol = tmpfile()) == NULL) {
           fprintf(stdout, "Error: cannot open temporary file\n");
           exit(1);
         }
         if (unZ(pFi, fol) < 10) {
           fprintf(stdout, "Error: cannot uncompress file\n");
           exit(1);
         }
         fclose(pFi);
         pFi = fol;
       }
       
      /* Read first line to infer file type */

      if (fgets(ligne, 255, pFi) == NULL) {
           fprintf(stdout, "Error reading 1st line\n");
           exit(1);
      }
      rewind(pFi);

      if (! strncmp(ligne, "HEADER", 6)) {

        p0 = LitStructure(pFi, argv[1], 0, PDB_FORMAT);
        if (p0 == (void*)BAD_POINTER || p0 == NULL) {
           fprintf(stdout, "Error: in reading PDB file\n");
           exit(1);
        }
        else if (RecodeAngles(p0, DEFAULT_ANGLE_MAX)==PASBON) {
           fprintf(stdout, "Error: in recoding protein angles\n");
           exit(1);
        }
        FreeProt(p0);
        fprintf(stdout,"PDB\n");
      }
      else if (ligne[0]=='>') {
   
         p0 = LitFormatFasta(pFi, 0, alphabet, 0);
         if ( p0 == (void*)BAD_POINTER || p0 == NULL) {
             fprintf(stdout, "Error: in reading fasta file\n");        
         }
         else {
             Upper(p0->tNameRes);
             if (checkSeq(p0->tNameRes,p0->nbRes,"ACGTNX")) {
		if (floor(fmod(p0->nbRes,3))!=0)
		   fprintf(stdout, "The length of your DNA sequence is not multiple of 3 ! Please do not select the p-value parameter.\n");
		else 
                   fprintf(stdout,"ADN\n");
                FreeProt(p0);
             }
             else if (checkSeq(p0->tNameRes,p0->nbRes,"ACDEFGHIKLMNPQRSTVWXY")) {
                fprintf(stdout,"PROTEIN\n");
                FreeProt(p0);
             }
             else {
                fprintf(stdout,"Error: unknown fasta sequence (unknown characters?)\n");
                FreeProt(p0);
             }
         }

      } 
      else {
         fprintf(stdout,"Error: unknown format file\n");
      }
        
      return(0);

}
