/* ---------------------------------------------------------------- */
/* Copyright (c) Atelier de BioInformatique			    */
/* @file: fasta_io.h						    */
/* @desc: sequence IO in fasta format / include file		    */
/*								    */
/* @history:							    */
/* @+	    <Gloup> : Aug 92 : first version			    */
/* @+       <Gloup> : Nov 95 : last revised version		    */
/* ---------------------------------------------------------------- */

#ifndef _H_fasta_io

#define _H_fasta_io

#ifndef _H_DTypes
#include "DTypes.h"		/* machine dependant types	*/
#endif

/* ==================================================== */
/* Constantes						*/
/* ==================================================== */

#define FASTA_NAMLEN  64  	/* max length of seq. name	 */
#define FASTA_COMLEN  512 	/* max length of seq. comment	 */

#define FASTA_CHAR_PER_LINE 50	/* # of chars per line in output */

/* ==================================================== */
/* Macros standards					*/
/* ==================================================== */

#ifndef NEW
#define NEW(typ)		(typ*)malloc(sizeof(typ)) 
#define NEWN(typ, dim) 		(typ*)malloc((unsigned)(dim) * sizeof(typ))
#define REALLOC(typ, ptr, dim)	(typ*)realloc((void *) (ptr), (unsigned long)(dim) * sizeof(typ))
#define NEWY(var, typ, dim)	{var=(typ*)malloc((size_t)(dim)*sizeof(typ));\
	                              if((var)==NULL){\
					fprintf(stderr,"pb malloc avec %s (%d %s)\n",\
					#var,__LINE__,__FILE__);exit(99);}} 
#define FREE(ptr)		free(ptr)
#endif

/* ==================================================== */
/* Structures de donnees				*/
/* ==================================================== */

typedef struct {			/* -- Sequence ---------------- */
	int	ok;			/* error flag			*/
	long 	length,			/* longueur			*/
		offset,			/* offset			*/
		bufsize;		/* size of current seq buffer	*/
	char    name[FASTA_NAMLEN],	/* nom 				*/
		comment[FASTA_COMLEN],	/* commentaire			*/
		*seq;			/* sequence			*/
} FastaSequence, *FastaSequencePtr;

/* ==================================================== */
/*  Prototypes (generated by mkproto)			*/
/* ==================================================== */

					/* libfasta.c 	*/

char  		 *GetFastaName 	    ( char *buffer );
char 		 *GetFastaComment   ( char *buffer );

FastaSequencePtr NewFastaSequence   ( void );
FastaSequencePtr FreeFastaSequence  ( FastaSequencePtr seq );

SBool		 ReadFastaSequence  ( FILE *streamin, FastaSequencePtr seq );
SBool		 GetFastaSequence   ( FILE *streamin, FastaSequencePtr seq );
void 		 WriteFastaSequence ( FILE *streamou, FastaSequencePtr seq , int char_per_line );

void		 RewindFastaDB	    ( FILE *streamin );

#endif
