/* ---------------------------------------------------------------- */
/* Copyright (c) Atelier de BioInformatique			    */
/* @file: fasta_io.h						    */
/* @desc: sequence IO in fasta format / include file		    */
/*								    */
/* @history:							    */
/* @+	    <Gloup> : Aug 92 : first version			    */
/* @+       <Gloup> : Nov 95 : last revised version		    */
/* ---------------------------------------------------------------- */

#ifndef _H_fasta_io

#define _H_fasta_io

#ifndef _H_Gtypes
#include "Gtypes.h"		/* machine dependant types	*/
#endif

/* ==================================================== */
/* Constantes						*/
/* ==================================================== */

#define FASTA_NAMLEN  150  	/* max length of seq. name	 */
#define FASTA_COMLEN  200  	/* max length of seq. comment	 */

#define FASTA_CHAR_PER_LINE 50	/* # of chars per line in output */



#define ONLY_SEQ	1	/* for WriteFastaSequence to     */
				/* write sequence only		 */

#define COMPLETE_ENTRY	 !(ONLY_SEQ)   	/* for WriteFastaSequence to 	 */
				/* write complete entry		 */
/* ==================================================== */
/* Macros standards					*/
/* ==================================================== */

#ifndef NEW
#define NEW(typ)		(typ*)malloc(sizeof(typ)) 
#define NEWN(typ, dim) 		(typ*)malloc((unsigned)(dim) * sizeof(typ))
#define REALLOC(typ, ptr, dim)	(typ*)realloc((void *) (ptr), (unsigned long)(dim) * sizeof(typ))
#define NEWY(var, typ, dim)	{var=(typ*)malloc((size_t)(dim)*sizeof(typ));\
	                              if((var)==NULL){\
					fprintf(stderr,"pb malloc avec %s (%d %s)\n",\
					#var,__LINE__,__FILE__);exit(99);}} 
#define FREE(ptr)		free(ptr)

#define FREE(ptr)		free(ptr)
#endif

/* ==================================================== */
/* Structures de donnees				*/
/* ==================================================== */

typedef struct {			/* -- Sequence ---------------- */
	int	ok;			/* error flag			*/
	long 	length,			/* longueur			*/
		offset,			/* offset			*/
		bufsize;		/* size of current seq buffer	*/
	char    name[FASTA_NAMLEN],	/* nom 				*/
		comment[FASTA_COMLEN],	/* commentaire			*/
		*seq;			/* sequence			*/
} FastaSequence, *FastaSequencePtr;

/* ==================================================== */
/*  Prototypes (generated by mkproto)			*/
/* ==================================================== */

					/* libfasta.c 	*/
char  		 *GetFastaName 	    P(( char *buffer ));
char 		 *GetFastaComment   P(( char *buffer ));

FastaSequencePtr NewFastaSequence   P(( void ));
FastaSequencePtr FreeFastaSequence  P(( FastaSequencePtr seq ));

Bool		 ReadFastaSequence  P(( FILE *streamin, FastaSequencePtr seq, int num ));
Bool		 GetFastaSequence   P(( FILE *streamin, FastaSequencePtr seq, int num));
void 		 WriteFastaSequence P(( FILE *streamou, FastaSequencePtr seq , int char_per_line, int only_seq ));

void		 RewindFastaDB	    P(( FILE *streamin, int num ));

#endif
